@@ -20,7 +20,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mike@dragonflylogic.com
 */
 
-/* $Id: dmtx.h 185 2008-06-17 20:07:47Z mblaughton $ */
+/* $Id: dmtx.h 519 2008-11-21 05:27:42Z mblaughton $ */
 
 /**
  * @file dmtx.h
@@ -30,6 +30,12 @@ Contact: mike@dragonflylogic.com
 #ifndef __DMTX_H__
 #define __DMTX_H__
 
+#include <time.h>
+
+#if defined(HAVE_SYS_TIME_H)
+#include <sys/time.h>
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -42,11 +48,16 @@ extern "C" {
 #define M_PI_2     1.57079632679489661923
 #endif
 
-#define DMTX_VERSION                   "0.5.1"
+#define DMTX_VERSION                   "0.6.0"
 
 #define DMTX_FAILURE                   0
 #define DMTX_SUCCESS                   1
 
+#define DMTX_FALSE                     0
+#define DMTX_TRUE                      1
+
+#define DMTX_BAD_OFFSET               -1
+
 #define DMTX_STATUS_NOT_SCANNED        0
 #define DMTX_STATUS_VALID              1
 #define DMTX_STATUS_INVALID            2
@@ -59,13 +70,19 @@ extern "C" {
 #define DMTX_REGION_NOT_FOUND          1
 #define DMTX_REGION_TIMEOUT            2
 #define DMTX_REGION_EOF                3
+#define DMTX_REGION_DROPPED_EDGE       4
+#define DMTX_REGION_DROPPED_FINDER     5
+#define DMTX_REGION_DROPPED_TOP        6
+#define DMTX_REGION_DROPPED_RIGHT      7
+#define DMTX_REGION_DROPPED_SIZE       8
 
 #define DMTX_MODULE_OFF             0x00
 #define DMTX_MODULE_ON_RED          0x01
 #define DMTX_MODULE_ON_GREEN        0x02
 #define DMTX_MODULE_ON_BLUE         0x04
 #define DMTX_MODULE_ON_RGB          0x07  /* ON_RED | ON_GREEN | ON_BLUE */
-#define DMTX_MODULE_UNSURE          0x08  /* ON_RED | ON_GREEN | ON_BLUE */
+#define DMTX_MODULE_ON              0x07
+#define DMTX_MODULE_UNSURE          0x08
 #define DMTX_MODULE_ASSIGNED        0x10
 #define DMTX_MODULE_VISITED         0x20
 #define DMTX_MODULE_DATA            0x40
@@ -73,9 +90,7 @@ extern "C" {
 #define DMTX_FORMAT_MATRIX             0
 #define DMTX_FORMAT_MOSAIC             1
 
-#define DMTX_SYMBOL_SQUARE_AUTO       -1
 #define DMTX_SYMBOL_SQUARE_COUNT      24
-#define DMTX_SYMBOL_RECT_AUTO         -2
 #define DMTX_SYMBOL_RECT_COUNT         6
 
 typedef enum {
@@ -127,13 +142,10 @@ typedef enum {
    DmtxSymAttribMappingMatrixRows,
    DmtxSymAttribMappingMatrixCols,
    DmtxSymAttribInterleavedBlocks,
-   DmtxSymAttribBlockDataWords,
    DmtxSymAttribBlockErrorWords,
-   DmtxSymAttribBlockTotalWords,
    DmtxSymAttribBlockMaxCorrectable,
    DmtxSymAttribSymbolDataWords,
    DmtxSymAttribSymbolErrorWords,
-   DmtxSymAttribSymbolTotalWords,
    DmtxSymAttribSymbolMaxCorrectable
 } DmtxSymAttribute;
 
@@ -144,6 +156,68 @@ typedef enum {
    DmtxCorner01 = 0x01 << 3
 } DmtxCornerLoc;
 
+typedef enum {
+   DmtxPropEdgeMin,
+   DmtxPropEdgeMax,
+   DmtxPropScanGap,
+   DmtxPropSquareDevn,
+   DmtxPropSymbolSize,
+   DmtxPropEdgeThresh,
+   DmtxPropWidth,
+   DmtxPropHeight,
+   DmtxPropArea,
+   DmtxPropXmin,
+   DmtxPropXmax,
+   DmtxPropYmin,
+   DmtxPropYmax,
+   DmtxPropShrinkMin,
+   DmtxPropShrinkMax,
+   DmtxPropScale,
+   DmtxPropScaledWidth,
+   DmtxPropScaledHeight,
+   DmtxPropScaledArea,
+   DmtxPropScaledXmin,
+   DmtxPropScaledXmax,
+   DmtxPropScaledYmin,
+   DmtxPropScaledYmax
+} DmtxDecodeProperty;
+
+typedef enum {
+   DmtxSymbolRectAuto   = -3,
+   DmtxSymbolSquareAuto = -2,
+   DmtxSymbolShapeAuto  = -1,
+   DmtxSymbol10x10      =  0,
+   DmtxSymbol12x12,
+   DmtxSymbol14x14,
+   DmtxSymbol16x16,
+   DmtxSymbol18x18,
+   DmtxSymbol20x20,
+   DmtxSymbol22x22,
+   DmtxSymbol24x24,
+   DmtxSymbol26x26,
+   DmtxSymbol32x32,
+   DmtxSymbol36x36,
+   DmtxSymbol40x40,
+   DmtxSymbol44x44,
+   DmtxSymbol48x48,
+   DmtxSymbol52x52,
+   DmtxSymbol64x64,
+   DmtxSymbol72x72,
+   DmtxSymbol80x80,
+   DmtxSymbol88x88,
+   DmtxSymbol96x96,
+   DmtxSymbol104x104,
+   DmtxSymbol120x120,
+   DmtxSymbol132x132,
+   DmtxSymbol144x144,
+   DmtxSymbol8x18,
+   DmtxSymbol8x32,
+   DmtxSymbol12x26,
+   DmtxSymbol12x36,
+   DmtxSymbol16x36,
+   DmtxSymbol16x48
+} DmtxSymbolSize;
+
 typedef double DmtxMatrix3[3][3];
 typedef unsigned char DmtxRgb[3];
 
@@ -164,6 +238,7 @@ typedef struct DmtxColor3_struct {
 typedef struct DmtxPixelLoc_struct {
    int X;
    int Y;
+   int status;
 } DmtxPixelLoc;
 
 /**
@@ -200,77 +275,63 @@ typedef struct DmtxRay2_struct {
  * @brief DmtxGradient
  */
 typedef struct DmtxGradient_struct {
-   char        isDefined;
-   double      tMin, tMax, tMid;
-   DmtxRay3    ray;
-   DmtxColor3 color, colorPrev; /* XXX maybe these aren't appropriate variables for a gradient? */
+   char       isDefined;
+   double     tMin, tMax;
+   DmtxRay3   ray;
 } DmtxGradient;
 
-typedef enum {
-   DmtxCompassDirNeg45 = 0x01,
-   DmtxCompassDir0     = 0x02,
-   DmtxCompassDir45    = 0x04,
-   DmtxCompassDir90    = 0x08
-} DmtxCompassDir;
-
-/**
- * @struct DmtxCompassEdge
- * @brief DmtxCompassEdge
- */
-typedef struct DmtxCompassEdge_struct {
-   char            assigned;
-   double          magnitude;  /* sqrt(R^2 + G^2 + B^2) */
-   DmtxColor3      intensity;
-   DmtxCompassDir  edgeDir;
-   DmtxCompassDir  scanDir;    /* DmtxCompassDir0 | DmtxCompassDir90 */
-} DmtxCompassEdge;
-
 /**
  * @struct DmtxImage
  * @brief DmtxImage
  */
 typedef struct DmtxImage_struct {
-   unsigned int    pageCount;
-   unsigned int    width;
-   unsigned int    height;
+   int             width;  /* unscaled */
+   int             height; /* unscaled */
+   int             xMin;   /* unscaled */
+   int             xMax;   /* unscaled */
+   int             yMin;   /* unscaled */
+   int             yMax;   /* unscaled */
+   int             scale;
+   int             widthScaled;
+   int             heightScaled;
+   int             xMinScaled;
+   int             xMaxScaled;
+   int             yMinScaled;
+   int             yMaxScaled;
+   int             pageCount;
+   unsigned char   *cache;
    DmtxRgb         *pxl;
-   DmtxCompassEdge *compass;
 } DmtxImage;
 
 /**
- * @struct DmtxEdge
- * @brief DmtxEdge
+ * @struct DmtxPointFlow
+ * @brief DmtxPointFlow
  */
-typedef struct DmtxEdge_struct {
-   int        offset;
-   double     t;
-   DmtxColor3 color;
-} DmtxEdge;
+typedef struct DmtxPointFlow_struct {
+   int plane;
+   int arrive;
+   int depart;
+   int mag;
+   DmtxPixelLoc loc;
+} DmtxPointFlow;
 
 /**
- * @struct DmtxChain
- * @brief DmtxChain
+ * @struct DmtxBestLine
+ * @brief DmtxBestLine
  */
-typedef struct DmtxChain_struct {
-   double tx, ty;
-   double phi, shx;
-   double scx, scy;
-   double bx0, bx1;
-   double by0, by1;
-   double sz;
-} DmtxChain;
-
-/**
- * @struct DmtxCorners
- * @brief DmtxCorners
- */
-typedef struct DmtxCorners_struct {
-   DmtxCornerLoc known; /* combination of (DmtxCorner00 | DmtxCorner10 | DmtxCorner11 | DmtxCorner01) */
-   DmtxVector2 c00;
-   DmtxVector2 c10;
-   DmtxVector2 c11;
-   DmtxVector2 c01;
-} DmtxCorners;
+typedef struct DmtxBestLine_struct {
+   int angle;
+   int hOffset;
+   int mag;
+   int stepBeg;
+   int stepPos;
+   int stepNeg;
+   int distSq;
+   double devn;
+   DmtxPixelLoc locBeg;
+   DmtxPixelLoc locPos;
+   DmtxPixelLoc locNeg;
+} DmtxBestLine;
 
 /**
  * @struct DmtxRegion
@@ -278,16 +339,52 @@ typedef struct DmtxCorners_struct {
  */
 typedef struct DmtxRegion_struct {
    int             found;         /* DMTX_REGION_FOUND | DMTX_REGION_NOT_FOUND | DMTX_REGION_EOF */
-   DmtxGradient    gradient;      /* Linear blend of colors between background and symbol color */
-   DmtxChain       chain;         /* List of values that are used to build a transformation matrix */
-   DmtxCorners     corners;       /* Corners of barcode region */
-   DmtxMatrix3     raw2fit;       /* 3x3 transformation from raw image to fitted barcode grid */
-   DmtxMatrix3     fit2raw;       /* 3x3 transformation from fitted barcode grid to raw image */
+
+   /* Trail blazing values */
+   int             jumpToPos;     /* */
+   int             jumpToNeg;     /* */
+   int             stepsTotal;    /* */
+   DmtxPixelLoc    finalPos;      /* */
+   DmtxPixelLoc    finalNeg;      /* */
+   DmtxPixelLoc    boundMin;      /* */
+   DmtxPixelLoc    boundMax;      /* */
+   DmtxPointFlow   flowBegin;     /* */
+
+   /* Orientation values */
+   int             polarity;      /* */
+   int             stepR;
+   int             stepT;
+   DmtxPixelLoc    locR;          /* remove if stepR works above */
+   DmtxPixelLoc    locT;          /* remove if stepT works above */
+
+   /* Region fitting values */
+   int             leftKnown;     /* known == 1; unknown == 0 */
+   int             leftAngle;     /* hough angle of left edge */
+   DmtxPixelLoc    leftLoc;       /* known (arbitrary) location on left edge */
+   DmtxBestLine    leftLine;      /* */
+   int             bottomKnown;   /* known == 1; unknown == 0 */
+   int             bottomAngle;   /* hough angle of bottom edge */
+   DmtxPixelLoc    bottomLoc;     /* known (arbitrary) location on bottom edge */
+   DmtxBestLine    bottomLine;    /* */
+   int             topKnown;      /* known == 1; unknown == 0 */
+   int             topAngle;      /* hough angle of top edge */
+   DmtxPixelLoc    topLoc;        /* known (arbitrary) location on top edge */
+   int             rightKnown;    /* known == 1; unknown == 0 */
+   int             rightAngle;    /* hough angle of right edge */
+   DmtxPixelLoc    rightLoc;      /* known (arbitrary) location on right edge */
+
+   /* Region calibration values */
+   int             onColor;       /* */
+   int             offColor;      /* */
    int             sizeIdx;       /* Index of arrays that store Data Matrix constants */
    int             symbolRows;    /* Number of total rows in symbol including alignment patterns */
    int             symbolCols;    /* Number of total columns in symbol including alignment patterns */
    int             mappingRows;   /* Number of data rows in symbol */
    int             mappingCols;   /* Number of data columns in symbol */
+
+   /* Transform values */
+   DmtxMatrix3     raw2fit;       /* 3x3 transformation from raw image to fitted barcode grid */
+   DmtxMatrix3     fit2raw;       /* 3x3 transformation from fitted barcode grid to raw image */
 } DmtxRegion;
 
 /**
@@ -314,6 +411,10 @@ typedef struct DmtxScanGrid_struct {
    int maxExtent;  /* Size of bounding grid region (2^N - 1) */
    int xOffset;    /* Offset to obtain image X coordinate */
    int yOffset;    /* Offset to obtain image Y coordinate */
+   int xMin;       /* Minimum X in image coordinate system */
+   int xMax;       /* Maximum X in image coordinate system */
+   int yMin;       /* Minimum Y in image coordinate system */
+   int yMax;       /* Maximum Y in image coordinate system */
 
    /* reset for each level */
    int total;      /* Total number of crosses at this size */
@@ -333,8 +434,16 @@ typedef struct DmtxScanGrid_struct {
  * @brief DmtxDecode
  */
 typedef struct DmtxDecode_struct {
-   DmtxImage *image;
-   DmtxScanGrid grid;
+   DmtxImage       *image;
+   DmtxScanGrid    grid;
+   int             edgeMin;
+   int             edgeMax;
+   int             scanGap;
+   double          squareDevn;
+   int             sizeIdxExpected;
+   int             edgeThresh;
+   int             shrinkMin;
+   int             shrinkMax;
 } DmtxDecode;
 
 /**
@@ -358,7 +467,7 @@ typedef struct DmtxEncode_struct {
    DmtxMessage *message;
    DmtxImage *image;
    DmtxRegion region;
-   DmtxMatrix3 xfrm; /* XXX still necessary? */
+   DmtxMatrix3 xfrm;  /* XXX still necessary? */
    DmtxMatrix3 rxfrm; /* XXX still necessary? */
 } DmtxEncode;
 
@@ -414,39 +523,50 @@ extern int dmtxEncodeDataMatrix(DmtxEncode *enc, int n, unsigned char *s, int si
 extern int dmtxEncodeDataMosaic(DmtxEncode *enc, int n, unsigned char *s, int sizeIdxRequest);
 
 /* dmtxdecode.c */
-extern DmtxDecode dmtxDecodeStructInit(DmtxImage *img, DmtxPixelLoc p0, DmtxPixelLoc p1, int gap);
+extern DmtxDecode dmtxDecodeStructInit(DmtxImage *img);
 extern void dmtxDecodeStructDeInit(DmtxDecode *dec);
-extern DmtxMessage *dmtxDecodeMatrixRegion(DmtxDecode *dec, DmtxRegion *reg, int fix);
-extern DmtxMessage *dmtxDecodeMosaicRegion(DmtxDecode *dec, DmtxRegion *reg, int fix);
+extern int dmtxDecodeSetProp(DmtxDecode *dec, int prop, int value);
+extern DmtxMessage *dmtxDecodeMatrixRegion(DmtxImage *img, DmtxRegion *reg, int fix);
+extern DmtxMessage *dmtxDecodeMosaicRegion(DmtxImage *img, DmtxRegion *reg, int fix);
 extern DmtxMessage *dmtxMessageMalloc(int sizeIdx, int symbolFormat);
 extern void dmtxMessageFree(DmtxMessage **mesg);
 
 /* dmtxregion.c */
 extern DmtxRegion dmtxDecodeFindNextRegion(DmtxDecode *decode, DmtxTime *timeout);
-extern DmtxRegion dmtxScanPixel(DmtxDecode *decode, DmtxPixelLoc loc);
+extern DmtxRegion dmtxRegionScanPixel(DmtxDecode *decode, DmtxPixelLoc loc);
+extern int dmtxRegionUpdateCorners(DmtxDecode *dec, DmtxRegion *reg, DmtxVector2 p00,
+      DmtxVector2 p10, DmtxVector2 p11, DmtxVector2 p01);
+extern int dmtxRegionUpdateXfrms(DmtxDecode *dec, DmtxRegion *reg);
 
 /* dmtximage.c */
 extern DmtxImage *dmtxImageMalloc(int width, int height);
 extern int dmtxImageFree(DmtxImage **img);
-extern int dmtxImageGetWidth(DmtxImage *img);
-extern int dmtxImageGetHeight(DmtxImage *img);
-extern int dmtxImageGetOffset(DmtxImage *img, DmtxDirection dir, int lineNbr, int offset);
+extern int dmtxImageSetProp(DmtxImage *img, int prop, int value);
+extern int dmtxImageGetProp(DmtxImage *img, int prop);
+extern int dmtxImageGetOffset(DmtxImage *img, int x, int y);
+extern int dmtxImageSetRgb(DmtxImage *img, int x, int y, DmtxRgb rgb);
+extern int dmtxImageGetRgb(DmtxImage *img, int x, int y, DmtxRgb rgb);
+extern int dmtxImageGetColor(DmtxImage *img, int x, int y, int colorPlane);
+extern int dmtxImageContainsInt(DmtxImage *img, int margin, int x, int y);
+extern int dmtxImageContainsFloat(DmtxImage *img, double x, double y);
 
-extern DmtxVector2 *dmtxVector2AddTo(DmtxVector2 *v1, DmtxVector2 *v2);
-extern DmtxVector2 *dmtxVector2Add(DmtxVector2 *vOut, DmtxVector2 *v1, DmtxVector2 *v2);
-extern DmtxVector2 *dmtxVector2SubFrom(DmtxVector2 *v1, DmtxVector2 *v2);
-extern DmtxVector2 *dmtxVector2Sub(DmtxVector2 *vOut, DmtxVector2 *v1, DmtxVector2 *v2);
+/* dmtxvector2.c */
+extern DmtxVector2 *dmtxVector2AddTo(DmtxVector2 *v1, const DmtxVector2 *v2);
+extern DmtxVector2 *dmtxVector2Add(DmtxVector2 *vOut, const DmtxVector2 *v1, const DmtxVector2 *v2);
+extern DmtxVector2 *dmtxVector2SubFrom(DmtxVector2 *v1, const DmtxVector2 *v2);
+extern DmtxVector2 *dmtxVector2Sub(DmtxVector2 *vOut, const DmtxVector2 *v1, const DmtxVector2 *v2);
 extern DmtxVector2 *dmtxVector2ScaleBy(DmtxVector2 *v, double s);
-extern DmtxVector2 *dmtxVector2Scale(DmtxVector2 *vOut, DmtxVector2 *v, double s);
-extern double dmtxVector2Cross(DmtxVector2 *v1, DmtxVector2 *v2);
-extern int dmtxVector2Norm(DmtxVector2 *v);
-extern double dmtxVector2Dot(DmtxVector2 *v1, DmtxVector2 *v2);
-extern double dmtxVector2Mag(DmtxVector2 *v);
-extern double dmtxDistanceFromRay2(DmtxRay2 *r, DmtxVector2 *q);
-extern double dmtxDistanceAlongRay2(DmtxRay2 *r, DmtxVector2 *q);
-extern int dmtxRay2Intersect(DmtxVector2 *point, DmtxRay2 *p0, DmtxRay2 *p1);
-extern int dmtxPointAlongRay2(DmtxVector2 *point, DmtxRay2 *r, double t);
+extern DmtxVector2 *dmtxVector2Scale(DmtxVector2 *vOut, const DmtxVector2 *v, double s);
+extern double dmtxVector2Cross(const DmtxVector2 *v1, const DmtxVector2 *v2);
+extern double dmtxVector2Norm(DmtxVector2 *v);
+extern double dmtxVector2Dot(const DmtxVector2 *v1, const DmtxVector2 *v2);
+extern double dmtxVector2Mag(const DmtxVector2 *v);
+extern double dmtxDistanceFromRay2(const DmtxRay2 *r, const DmtxVector2 *q);
+extern double dmtxDistanceAlongRay2(const DmtxRay2 *r, const DmtxVector2 *q);
+extern int dmtxRay2Intersect(DmtxVector2 *point, const DmtxRay2 *p0, const DmtxRay2 *p1);
+extern int dmtxPointAlongRay2(DmtxVector2 *point, const DmtxRay2 *r, double t);
 
+/* dmtxmatrix3.c */
 extern void dmtxMatrix3Copy(DmtxMatrix3 m0, DmtxMatrix3 m1);
 extern void dmtxMatrix3Identity(DmtxMatrix3 m);
 extern void dmtxMatrix3Translate(DmtxMatrix3 m, double tx, double ty);
@@ -463,7 +583,7 @@ extern int dmtxMatrix3VMultiply(DmtxVector2 *vOut, DmtxVector2 *vIn, DmtxMatrix3
 extern int dmtxMatrix3VMultiplyBy(DmtxVector2 *v, DmtxMatrix3 m);
 extern void dmtxMatrix3Print(DmtxMatrix3 m);
 
-extern void dmtxPixelFromImage(DmtxRgb rgb, DmtxImage *img, int x, int y);
+/* dmtxcolor3.c */
 extern void dmtxColor3FromImage2(DmtxColor3 *color, DmtxImage *img, DmtxVector2 p);
 extern DmtxColor3 *dmtxColor3FromPixel(DmtxColor3 *color, DmtxRgb rgb);
 extern void dmtxPixelFromColor3(DmtxRgb rgb, DmtxColor3 *color);
@@ -475,15 +595,18 @@ extern DmtxColor3 *dmtxColor3Sub(DmtxColor3 *vOut, DmtxColor3 *v1, DmtxColor3 *v
 extern DmtxColor3 *dmtxColor3ScaleBy(DmtxColor3 *v, double s);
 extern DmtxColor3 *dmtxColor3Scale(DmtxColor3 *vOut, DmtxColor3 *v, double s);
 extern DmtxColor3 *dmtxColor3Cross(DmtxColor3 *vOut, DmtxColor3 *v1, DmtxColor3 *v2);
-extern int dmtxColor3Norm(DmtxColor3 *v);
+extern double dmtxColor3Norm(DmtxColor3 *v);
 extern double dmtxColor3Dot(DmtxColor3 *v1, DmtxColor3 *v2);
+extern double dmtxColor3MagSquared(DmtxColor3 *v);
 extern double dmtxColor3Mag(DmtxColor3 *v);
 extern double dmtxDistanceFromRay3(DmtxRay3 *r, DmtxColor3 *q);
 extern double dmtxDistanceAlongRay3(DmtxRay3 *r, DmtxColor3 *q);
 extern int dmtxPointAlongRay3(DmtxColor3 *point, DmtxRay3 *r, double t);
 
+/* dmtxsymbol.c */
 extern int dmtxSymbolModuleStatus(DmtxMessage *mapping, int sizeIdx, int row, int col);
 extern int dmtxGetSymbolAttribute(int attribute, int sizeIdx);
+extern int dmtxGetBlockDataSize(int sizeIdx, int blockIdx);
 
 extern char *dmtxVersion(void);
 
