@@ -20,7 +20,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mike@dragonflylogic.com
 */
 
-/* $Id: dmtxdecode.c 186 2008-06-20 19:17:27Z mblaughton $ */
+/* $Id: dmtxdecode.c 514 2008-11-19 17:10:44Z mblaughton $ */
 
 /**
  * @file dmtxdecode.c
@@ -28,30 +28,48 @@ Contact: mike@dragonflylogic.com
  */
 
 /**
- * @brief  XXX
- * @param  image
- * @param  p0
- * @param  p1
- * @param  minGapSize
+ * @brief  Initialize decode struct with default values
+ * @param  img
  * @return Initialized DmtxDecode struct
  */
 extern DmtxDecode
-dmtxDecodeStructInit(DmtxImage *image, DmtxPixelLoc p0, DmtxPixelLoc p1, int minGapSize)
+dmtxDecodeStructInit(DmtxImage *img)
 {
    DmtxDecode dec;
+   int cacheSize;
 
    memset(&dec, 0x00, sizeof(DmtxDecode));
 
-   dec.image = image;
-   memset(dec.image->compass, 0x00, image->height * image->width * sizeof(DmtxCompassEdge));
+   dec.image = img;
 
-   dec.grid = InitScanGrid(image, p0, p1, minGapSize);
+   cacheSize = dmtxImageGetProp(img, DmtxPropWidth) *
+         dmtxImageGetProp(img, DmtxPropHeight) * sizeof(unsigned char);
+
+   memset(dec.image->cache, 0x00, cacheSize);
+
+   /* These values should probably be stored in the decode struct */
+   img->scale = 1;
+   img->xMin = img->xMinScaled = 0;
+   img->xMax = img->xMaxScaled = img->width - 1;
+   img->yMin = img->yMinScaled = 0;
+   img->yMax = img->yMaxScaled = img->height - 1;
+
+   dec.edgeMin = -1;
+   dec.edgeMax = -1;
+   dec.scanGap = 1;
+   dec.squareDevn = cos(50 * (M_PI/180));
+   dec.sizeIdxExpected = DmtxSymbolShapeAuto;
+   dec.edgeThresh = 10;
+   dec.shrinkMin = 1;
+   dec.shrinkMax = 1;
+
+   dec.grid = InitScanGrid(img, dec.scanGap);
 
    return dec;
 }
 
 /**
- * @brief  XXX
+ * @brief  Deinitialize decode struct
  * @param  dec
  * @return void
  */
@@ -62,124 +80,228 @@ dmtxDecodeStructDeInit(DmtxDecode *dec)
 }
 
 /**
- * @brief  XXX
+ * @brief  Set decoding behavior property
  * @param  dec
- * @param  region
- * @param  fix
- * @return Decoded message
+ * @param  prop
+ * @param  value
+ * @return DMTX_SUCCESS | DMTX_FAILURE
  */
-extern DmtxMessage *
-dmtxDecodeMatrixRegion(DmtxDecode *dec, DmtxRegion *region, int fix)
+extern int
+dmtxDecodeSetProp(DmtxDecode *dec, int prop, int value)
 {
-   DmtxMessage *message;
+   int err;
 
-   message = dmtxMessageMalloc(region->sizeIdx, DMTX_FORMAT_MATRIX);
-   if(message == NULL)
-      return NULL;
-
-   if(PopulateArrayFromMatrix(message, dec->image, region) != DMTX_SUCCESS) {
-      dmtxMessageFree(&message);
-      return NULL;
+   switch(prop) {
+      case DmtxPropEdgeMin:
+         dec->edgeMin = value;
+         break;
+      case DmtxPropEdgeMax:
+         dec->edgeMax = value;
+         break;
+      case DmtxPropScanGap:
+         dec->scanGap = value;
+         break;
+      case DmtxPropSquareDevn:
+         dec->squareDevn = cos(value * (M_PI/180.0));
+         break;
+      case DmtxPropSymbolSize:
+         dec->sizeIdxExpected = value;
+         break;
+      case DmtxPropEdgeThresh:
+         dec->edgeThresh = value;
+         break;
+      case DmtxPropXmin:
+         err = dmtxImageSetProp(dec->image, DmtxPropXmin, value);
+         if(err == DMTX_FAILURE)
+            return DMTX_FAILURE;
+         break;
+      case DmtxPropXmax:
+         err = dmtxImageSetProp(dec->image, DmtxPropXmax, value);
+         if(err == DMTX_FAILURE)
+            return DMTX_FAILURE;
+         break;
+      case DmtxPropYmin:
+         err = dmtxImageSetProp(dec->image, DmtxPropYmin, value);
+         if(err == DMTX_FAILURE)
+            return DMTX_FAILURE;
+         break;
+      case DmtxPropYmax:
+         err = dmtxImageSetProp(dec->image, DmtxPropYmax, value);
+         if(err == DMTX_FAILURE)
+            return DMTX_FAILURE;
+         break;
+      case DmtxPropShrinkMin:
+         dec->shrinkMin = value;
+         break;
+      case DmtxPropShrinkMax:
+         dec->shrinkMax = value;
+         err = dmtxImageSetProp(dec->image, DmtxPropScale, value);
+         if(err == DMTX_FAILURE)
+            return DMTX_FAILURE;
+         break;
    }
 
-   ModulePlacementEcc200(message->array, message->code,
-         region->sizeIdx, DMTX_MODULE_ON_RED | DMTX_MODULE_ON_GREEN | DMTX_MODULE_ON_BLUE);
+   /* Minimum image scale can't be larger than maximum image scale */
+   if(dec->shrinkMin < 1 || dec->shrinkMax < dec->shrinkMin)
+      return DMTX_FAILURE;
 
-   if(DecodeCheckErrors(message->code, region->sizeIdx, fix) != DMTX_SUCCESS) {
-      dmtxMessageFree(&message);
-      return NULL;
-   }
+   if(dec->squareDevn <= 0.0 || dec->squareDevn >= 1.0)
+      return DMTX_FAILURE;
 
-   DecodeDataStream(message, region->sizeIdx, NULL);
+   if(dec->scanGap < 1)
+      return DMTX_FAILURE;
 
-   return message;
+   if(dec->edgeThresh < 1 || dec->edgeThresh > 100)
+      return DMTX_FAILURE;
+
+   /* Reinitialize scangrid if any inputs changed */
+   dec->grid = InitScanGrid(dec->image, dec->scanGap);
+
+   return DMTX_SUCCESS;
 }
 
 /**
- * @brief  XXX
+ * @brief  Convert fitted Data Matrix region into a decoded message
  * @param  dec
- * @param  region
+ * @param  reg
  * @param  fix
  * @return Decoded message
  */
 extern DmtxMessage *
-dmtxDecodeMosaicRegion(DmtxDecode *dec, DmtxRegion *region, int fix)
+dmtxDecodeMatrixRegion(DmtxImage *img, DmtxRegion *reg, int fix)
+{
+   int row, col;
+   int width, height;
+   int offset;
+   DmtxMessage *msg;
+   DmtxVector2 p;
+
+   msg = dmtxMessageMalloc(reg->sizeIdx, DMTX_FORMAT_MATRIX);
+   if(msg == NULL)
+      return NULL;
+
+   if(PopulateArrayFromMatrix(msg, img, reg) != DMTX_SUCCESS) {
+      dmtxMessageFree(&msg);
+      return NULL;
+   }
+
+   ModulePlacementEcc200(msg->array, msg->code,
+         reg->sizeIdx, DMTX_MODULE_ON_RED | DMTX_MODULE_ON_GREEN | DMTX_MODULE_ON_BLUE);
+
+   if(DecodeCheckErrors(msg->code, reg->sizeIdx, fix) != DMTX_SUCCESS) {
+      dmtxMessageFree(&msg);
+      return NULL;
+   }
+
+   width = dmtxImageGetProp(img, DmtxPropScaledWidth);
+   height = dmtxImageGetProp(img, DmtxPropScaledHeight);
+   for(row = 0; row < height; row++) {
+      for(col = 0; col < width; col++) {
+         p.X = col;
+         p.Y = row;
+         dmtxMatrix3VMultiplyBy(&p, reg->raw2fit);
+         /* XXX tighten these boundaries can by accounting for barcode size */
+         if(p.X >= -0.1 && p.X <= 1.1 && p.Y >= -0.1 && p.Y <= 1.1) {
+
+            offset = dmtxImageGetOffset(img, col, row);
+            if(offset == DMTX_BAD_OFFSET)
+               continue;
+            else
+               img->cache[offset] |= 0x80; /* Mark as visited */
+         }
+      }
+   }
+
+   DecodeDataStream(msg, reg->sizeIdx, NULL);
+
+   return msg;
+}
+
+/**
+ * @brief  Convert fitted Data Mosaic region into a decoded message
+ * @param  dec
+ * @param  reg
+ * @param  fix
+ * @return Decoded message
+ */
+extern DmtxMessage *
+dmtxDecodeMosaicRegion(DmtxImage *img, DmtxRegion *reg, int fix)
 {
    int row, col;
    int mappingRows, mappingCols;
-   DmtxMessage *message;
+   DmtxMessage *msg;
    DmtxMessage rMesg, gMesg, bMesg;
 
-   mappingRows = dmtxGetSymbolAttribute(DmtxSymAttribMappingMatrixRows, region->sizeIdx);
-   mappingCols = dmtxGetSymbolAttribute(DmtxSymAttribMappingMatrixCols, region->sizeIdx);
+   mappingRows = dmtxGetSymbolAttribute(DmtxSymAttribMappingMatrixRows, reg->sizeIdx);
+   mappingCols = dmtxGetSymbolAttribute(DmtxSymAttribMappingMatrixCols, reg->sizeIdx);
 
-   message = dmtxMessageMalloc(region->sizeIdx, DMTX_FORMAT_MOSAIC);
-   if(message == NULL)
+   msg = dmtxMessageMalloc(reg->sizeIdx, DMTX_FORMAT_MOSAIC);
+   if(msg == NULL)
       return NULL;
 
-   rMesg = gMesg = bMesg = *message;
-   rMesg.codeSize = gMesg.codeSize = bMesg.codeSize = message->codeSize/3;
+   rMesg = gMesg = bMesg = *msg;
+   rMesg.codeSize = gMesg.codeSize = bMesg.codeSize = msg->codeSize/3;
 
    gMesg.code += gMesg.codeSize;
    bMesg.code += (bMesg.codeSize * 2);
 
-   if(PopulateArrayFromMosaic(message, dec->image, region) != DMTX_SUCCESS) {
-      dmtxMessageFree(&message);
+   if(PopulateArrayFromMosaic(msg, img, reg) != DMTX_SUCCESS) {
+      dmtxMessageFree(&msg);
       return NULL;
    }
 
-   ModulePlacementEcc200(message->array, rMesg.code, region->sizeIdx, DMTX_MODULE_ON_RED);
-   if(DecodeCheckErrors(rMesg.code, region->sizeIdx, fix) != DMTX_SUCCESS) {
-      dmtxMessageFree(&message);
+   ModulePlacementEcc200(msg->array, rMesg.code, reg->sizeIdx, DMTX_MODULE_ON_RED);
+   if(DecodeCheckErrors(rMesg.code, reg->sizeIdx, fix) != DMTX_SUCCESS) {
+      dmtxMessageFree(&msg);
       return NULL;
    }
 
    for(row = 0; row < mappingRows; row++)
       for(col = 0; col < mappingCols; col++)
-         message->array[row*mappingCols+col] &= (0xff ^ DMTX_MODULE_VISITED);
+         msg->array[row*mappingCols+col] &= (0xff ^ DMTX_MODULE_VISITED);
 
-   ModulePlacementEcc200(message->array, gMesg.code, region->sizeIdx, DMTX_MODULE_ON_GREEN);
-   if(DecodeCheckErrors(gMesg.code, region->sizeIdx, fix) != DMTX_SUCCESS) {
-      dmtxMessageFree(&message);
+   ModulePlacementEcc200(msg->array, gMesg.code, reg->sizeIdx, DMTX_MODULE_ON_GREEN);
+   if(DecodeCheckErrors(gMesg.code, reg->sizeIdx, fix) != DMTX_SUCCESS) {
+      dmtxMessageFree(&msg);
       return NULL;
    }
 
    for(row = 0; row < mappingRows; row++)
       for(col = 0; col < mappingCols; col++)
-         message->array[row*mappingCols+col] &= (0xff ^ DMTX_MODULE_VISITED);
+         msg->array[row*mappingCols+col] &= (0xff ^ DMTX_MODULE_VISITED);
 
-   ModulePlacementEcc200(message->array, bMesg.code, region->sizeIdx, DMTX_MODULE_ON_BLUE);
-   if(DecodeCheckErrors(bMesg.code, region->sizeIdx, fix) != DMTX_SUCCESS) {
-      dmtxMessageFree(&message);
+   ModulePlacementEcc200(msg->array, bMesg.code, reg->sizeIdx, DMTX_MODULE_ON_BLUE);
+   if(DecodeCheckErrors(bMesg.code, reg->sizeIdx, fix) != DMTX_SUCCESS) {
+      dmtxMessageFree(&msg);
       return NULL;
    }
 
-   DecodeDataStream(&rMesg, region->sizeIdx, NULL);
-   DecodeDataStream(&gMesg, region->sizeIdx, rMesg.output + rMesg.outputIdx);
-   DecodeDataStream(&bMesg, region->sizeIdx, gMesg.output + gMesg.outputIdx);
+   DecodeDataStream(&rMesg, reg->sizeIdx, NULL);
+   DecodeDataStream(&gMesg, reg->sizeIdx, rMesg.output + rMesg.outputIdx);
+   DecodeDataStream(&bMesg, reg->sizeIdx, gMesg.output + gMesg.outputIdx);
 
-   message->outputIdx = rMesg.outputIdx + gMesg.outputIdx + bMesg.outputIdx;
+   msg->outputIdx = rMesg.outputIdx + gMesg.outputIdx + bMesg.outputIdx;
 
-   return message;
+   return msg;
 }
 
 /**
- * @brief  XXX
- * @param  message
+ * @brief  Translate encoded data stream into final output
+ * @param  msg
  * @param  sizeIdx
  * @param  outputStart
  * @return void
  */
 static void
-DecodeDataStream(DmtxMessage *message, int sizeIdx, unsigned char *outputStart)
+DecodeDataStream(DmtxMessage *msg, int sizeIdx, unsigned char *outputStart)
 {
    DmtxSchemeDecode encScheme;
    unsigned char *ptr, *dataEnd;
 
-   message->output = (outputStart == NULL) ? message->output : outputStart;
-   message->outputIdx = 0;
+   msg->output = (outputStart == NULL) ? msg->output : outputStart;
+   msg->outputIdx = 0;
 
-   ptr = message->code;
+   ptr = msg->code;
    dataEnd = ptr + dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, sizeIdx);
 
    while(ptr < dataEnd) {
@@ -188,35 +310,35 @@ DecodeDataStream(DmtxMessage *message, int sizeIdx, unsigned char *outputStart)
 
       switch(encScheme) {
          case DmtxSchemeDecodeAsciiStd:
-            ptr = DecodeSchemeAsciiStd(message, ptr, dataEnd);
+            ptr = DecodeSchemeAsciiStd(msg, ptr, dataEnd);
             break;
 
          case DmtxSchemeDecodeAsciiExt:
-            ptr = DecodeSchemeAsciiExt(message, ptr, dataEnd);
+            ptr = DecodeSchemeAsciiExt(msg, ptr, dataEnd);
             break;
 
          case DmtxSchemeDecodeC40:
          case DmtxSchemeDecodeText:
-            ptr = DecodeSchemeC40Text(message, ptr, dataEnd, encScheme);
+            ptr = DecodeSchemeC40Text(msg, ptr, dataEnd, encScheme);
             break;
 
          case DmtxSchemeDecodeX12:
-            ptr = DecodeSchemeX12(message, ptr, dataEnd);
+            ptr = DecodeSchemeX12(msg, ptr, dataEnd);
             break;
 
          case DmtxSchemeDecodeEdifact:
-            ptr = DecodeSchemeEdifact(message, ptr, dataEnd);
+            ptr = DecodeSchemeEdifact(msg, ptr, dataEnd);
             break;
 
          case DmtxSchemeDecodeBase256:
-            ptr = DecodeSchemeBase256(message, ptr, dataEnd);
+            ptr = DecodeSchemeBase256(msg, ptr, dataEnd);
             break;
       }
    }
 }
 
 /**
- * @brief  XXX
+ * @brief  Determine next encodation scheme
  * @param  encScheme
  * @param  ptr
  * @return Pointer to next undecoded codeword
@@ -252,57 +374,57 @@ NextEncodationScheme(DmtxSchemeDecode *encScheme, unsigned char *ptr)
 }
 
 /**
- * @brief  XXX
- * @param  message
+ * @brief  Decode stream assuming standard ASCII encodation
+ * @param  msg
  * @param  ptr
  * @param  dataEnd
  * @return Pointer to next undecoded codeword
  */
 static unsigned char *
-DecodeSchemeAsciiStd(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd)
+DecodeSchemeAsciiStd(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd)
 {
    int digits;
 
    if(*ptr <= 128) {
-      message->output[message->outputIdx++] = *ptr - 1;
+      msg->output[msg->outputIdx++] = *ptr - 1;
    }
    else if(*ptr == 129) {
       return dataEnd;
    }
    else if(*ptr <= 229) {
       digits = *ptr - 130;
-      message->output[message->outputIdx++] = digits/10 + '0';
-      message->output[message->outputIdx++] = digits - (digits/10)*10 + '0';
+      msg->output[msg->outputIdx++] = digits/10 + '0';
+      msg->output[msg->outputIdx++] = digits - (digits/10)*10 + '0';
    }
 
    return ptr + 1;
 }
 
 /**
- * @brief  XXX
- * @param  message
+ * @brief  Decode stream assuming extended ASCII encodation
+ * @param  msg
  * @param  ptr
  * @param  dataEnd
  * @return Pointer to next undecoded codeword
  */
 static unsigned char *
-DecodeSchemeAsciiExt(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd)
+DecodeSchemeAsciiExt(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd)
 {
-   message->output[message->outputIdx++] = *ptr + 128;
+   msg->output[msg->outputIdx++] = *ptr + 128;
 
    return ptr + 1;
 }
 
 /**
- * @brief  XXX
- * @param  message
+ * @brief  Decode stream assuming C40 or Text encodation
+ * @param  msg
  * @param  ptr
  * @param  dataEnd
  * @param  encScheme
  * @return Pointer to next undecoded codeword
  */
 static unsigned char *
-DecodeSchemeC40Text(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd, DmtxSchemeDecode encScheme)
+DecodeSchemeC40Text(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd, DmtxSchemeDecode encScheme)
 {
    int i;
    int packed;
@@ -326,32 +448,32 @@ DecodeSchemeC40Text(DmtxMessage *message, unsigned char *ptr, unsigned char *dat
                shift = c40Values[i] + 1;
             }
             else if(c40Values[i] == 3) {
-               message->output[message->outputIdx++] = ' '; /* Space */
+               msg->output[msg->outputIdx++] = ' '; /* Space */
             }
             else if(c40Values[i] <= 13) {
-               message->output[message->outputIdx++] = c40Values[i] - 13 + '9'; /* 0-9 */
+               msg->output[msg->outputIdx++] = c40Values[i] - 13 + '9'; /* 0-9 */
             }
             else if(c40Values[i] <= 39) {
                if(encScheme == DmtxSchemeDecodeC40) {
-                  message->output[message->outputIdx++] = c40Values[i] - 39 + 'Z'; /* A-Z */
+                  msg->output[msg->outputIdx++] = c40Values[i] - 39 + 'Z'; /* A-Z */
                }
                else if(encScheme == DmtxSchemeDecodeText) {
-                  message->output[message->outputIdx++] = c40Values[i] - 39 + 'z'; /* a-z */
+                  msg->output[msg->outputIdx++] = c40Values[i] - 39 + 'z'; /* a-z */
                }
             }
          }
          else if(shift == 1) { /* Shift 1 set */
-            message->output[message->outputIdx++] = c40Values[i]; /* ASCII 0 - 31 */
+            msg->output[msg->outputIdx++] = c40Values[i]; /* ASCII 0 - 31 */
 
             shift = 0;
          }
          else if(shift == 2) { /* Shift 2 set */
             if(c40Values[i] <= 14)
-               message->output[message->outputIdx++] = c40Values[i] + 33; /* ASCII 33 - 47 */
+               msg->output[msg->outputIdx++] = c40Values[i] + 33; /* ASCII 33 - 47 */
             else if(c40Values[i] <= 21)
-               message->output[message->outputIdx++] = c40Values[i] + 43; /* ASCII 58 - 64 */
+               msg->output[msg->outputIdx++] = c40Values[i] + 43; /* ASCII 58 - 64 */
             else if(c40Values[i] <= 26)
-               message->output[message->outputIdx++] = c40Values[i] + 69; /* ASCII 91 - 95 */
+               msg->output[msg->outputIdx++] = c40Values[i] + 69; /* ASCII 91 - 95 */
             else if(c40Values[i] == 27)
                fprintf(stdout, "FNC1 (?)"); /* FNC1 (eh?) */
             else if(c40Values[i] == 30)
@@ -361,15 +483,15 @@ DecodeSchemeC40Text(DmtxMessage *message, unsigned char *ptr, unsigned char *dat
          }
          else if(shift == 3) { /* Shift 3 set */
             if(encScheme == DmtxSchemeDecodeC40) {
-               message->output[message->outputIdx++] = c40Values[i] + 96;
+               msg->output[msg->outputIdx++] = c40Values[i] + 96;
             }
             else if(encScheme == DmtxSchemeDecodeText) {
                if(c40Values[i] == 0)
-                  message->output[message->outputIdx++] = c40Values[i] + 96;
+                  msg->output[msg->outputIdx++] = c40Values[i] + 96;
                else if(c40Values[i] <= 26)
-                  message->output[message->outputIdx++] = c40Values[i] - 26 + 'Z'; /* A-Z */
+                  msg->output[msg->outputIdx++] = c40Values[i] - 26 + 'Z'; /* A-Z */
                else
-                  message->output[message->outputIdx++] = c40Values[i] - 31 + 127; /* { | } ~ DEL */
+                  msg->output[msg->outputIdx++] = c40Values[i] - 31 + 127; /* { | } ~ DEL */
             }
 
             shift = 0;
@@ -389,14 +511,14 @@ DecodeSchemeC40Text(DmtxMessage *message, unsigned char *ptr, unsigned char *dat
 }
 
 /**
- * @brief  XXX
- * @param  message
+ * @brief  Decode stream assuming X12 encodation
+ * @param  msg
  * @param  ptr
  * @param  dataEnd
  * @return Pointer to next undecoded codeword
  */
 static unsigned char *
-DecodeSchemeX12(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd)
+DecodeSchemeX12(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd)
 {
    int i;
    int packed;
@@ -413,17 +535,17 @@ DecodeSchemeX12(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd
 
       for(i = 0; i < 3; i++) {
          if(x12Values[i] == 0)
-            message->output[message->outputIdx++] = 13;
+            msg->output[msg->outputIdx++] = 13;
          else if(x12Values[i] == 1)
-            message->output[message->outputIdx++] = 42;
+            msg->output[msg->outputIdx++] = 42;
          else if(x12Values[i] == 2)
-            message->output[message->outputIdx++] = 62;
+            msg->output[msg->outputIdx++] = 62;
          else if(x12Values[i] == 3)
-            message->output[message->outputIdx++] = 32;
+            msg->output[msg->outputIdx++] = 32;
          else if(x12Values[i] <= 13)
-            message->output[message->outputIdx++] = x12Values[i] + 44;
+            msg->output[msg->outputIdx++] = x12Values[i] + 44;
          else if(x12Values[i] <= 90)
-            message->output[message->outputIdx++] = x12Values[i] + 51;
+            msg->output[msg->outputIdx++] = x12Values[i] + 51;
       }
 
       /* Unlatch if codeword 254 follows 2 codewords in C40/Text encodation */
@@ -439,14 +561,14 @@ DecodeSchemeX12(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd
 }
 
 /**
- * @brief  XXX
- * @param  message
+ * @brief  Decode stream assuming EDIFACT encodation
+ * @param  msg
  * @param  ptr
  * @param  dataEnd
  * @return Pointer to next undecoded codeword
  */
 static unsigned char *
-DecodeSchemeEdifact(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd)
+DecodeSchemeEdifact(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd)
 {
    int i;
    unsigned char unpacked[4];
@@ -469,11 +591,11 @@ DecodeSchemeEdifact(DmtxMessage *message, unsigned char *ptr, unsigned char *dat
 
          /* Test for unlatch condition */
          if(unpacked[i] == 0x1f) {
-            assert(message->output[message->outputIdx] == 0); /* XXX dirty why? */
+            assert(msg->output[msg->outputIdx] == 0); /* XXX dirty why? */
             return ptr;
          }
 
-         message->output[message->outputIdx++] = unpacked[i] ^ (((unpacked[i] & 0x20) ^ 0x20) << 1);
+         msg->output[msg->outputIdx++] = unpacked[i] ^ (((unpacked[i] & 0x20) ^ 0x20) << 1);
       }
 
       /* Unlatch is implied if fewer than 3 codewords remain */
@@ -504,7 +626,7 @@ DecodeSchemeEdifact(DmtxMessage *message, unsigned char *ptr, unsigned char *dat
          assert(bits == 0); // should be padded with zero-value bits
          return ptr;
       }
-      message->output[message->outputIdx++] = value ^ (((value & 0x20) ^ 0x20) << 1);
+      msg->output[msg->outputIdx++] = value ^ (((value & 0x20) ^ 0x20) << 1);
 
       // Unlatch implied if just completed triplet and 1 or 2 words are left
       if(bitCount == 0 && dataEnd - ptr - 1 > 0 && dataEnd - ptr - 1 < 3)
@@ -518,21 +640,21 @@ DecodeSchemeEdifact(DmtxMessage *message, unsigned char *ptr, unsigned char *dat
 }
 
 /**
- * @brief  XXX
- * @param  message
+ * @brief  Decode stream assuming Base 256 encodation
+ * @param  msg
  * @param  ptr
  * @param  dataEnd
  * @return Pointer to next undecoded codeword
  */
 static unsigned char *
-DecodeSchemeBase256(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd)
+DecodeSchemeBase256(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd)
 {
    int d0, d1;
    int i;
    unsigned char *ptrEnd;
 
    /* XXX i is the positional index used for unrandomizing */
-   i = ptr - message->code + 1;
+   i = ptr - msg->code + 1;
 
    d0 = UnRandomize255State(*(ptr++), i++);
    if(d0 == 0) {
@@ -551,16 +673,16 @@ DecodeSchemeBase256(DmtxMessage *message, unsigned char *ptr, unsigned char *dat
    }
 
    while(ptr < ptrEnd) {
-      message->output[message->outputIdx++] = UnRandomize255State(*(ptr++), i++);
+      msg->output[msg->outputIdx++] = UnRandomize255State(*(ptr++), i++);
    }
 
    return ptr;
 }
 
 /**
- *
- * @param XXX
- * @return XXX
+ * @brief  Unrandomize 253 state
+ * @param  codewordValue
+ * @return codewordPosition
  */
 /**
 static unsigned char
@@ -577,7 +699,7 @@ UnRandomize253State(unsigned char codewordValue, int codewordPosition)
 */
 
 /**
- * @brief  XXX
+ * @brief  Unrandomize 255 state
  * @param  value
  * @param  idx
  * @return Unrandomized value
@@ -595,14 +717,14 @@ UnRandomize255State(unsigned char value, int idx)
 }
 
 /**
- * @brief  XXX
- * @param  message
- * @param  image
- * @param  region
+ * @brief  Populate array with codeword values based on module colors
+ * @param  msg
+ * @param  img
+ * @param  reg
  * @return DMTX_SUCCESS | DMTX_FAILURE
  */
 static int
-PopulateArrayFromMatrix(DmtxMessage *message, DmtxImage *image, DmtxRegion *region)
+PopulateArrayFromMatrix(DmtxMessage *msg, DmtxImage *img, DmtxRegion *reg)
 {
    int weightFactor;
    int mapWidth, mapHeight;
@@ -613,15 +735,15 @@ PopulateArrayFromMatrix(DmtxMessage *message, DmtxImage *image, DmtxRegion *regi
    int colTmp, rowTmp, idx;
    int tally[24][24]; /* Large enough to map largest single region */
 
-/* memset(message->array, 0x00, message->arraySize); */
+/* memset(msg->array, 0x00, msg->arraySize); */
 
    /* Capture number of regions present in barcode */
-   xRegionTotal = dmtxGetSymbolAttribute(DmtxSymAttribHorizDataRegions, region->sizeIdx);
-   yRegionTotal = dmtxGetSymbolAttribute(DmtxSymAttribVertDataRegions, region->sizeIdx);
+   xRegionTotal = dmtxGetSymbolAttribute(DmtxSymAttribHorizDataRegions, reg->sizeIdx);
+   yRegionTotal = dmtxGetSymbolAttribute(DmtxSymAttribVertDataRegions, reg->sizeIdx);
 
    /* Capture region dimensions (not including border modules) */
-   mapWidth = dmtxGetSymbolAttribute(DmtxSymAttribDataRegionCols, region->sizeIdx);
-   mapHeight = dmtxGetSymbolAttribute(DmtxSymAttribDataRegionRows, region->sizeIdx);
+   mapWidth = dmtxGetSymbolAttribute(DmtxSymAttribDataRegionCols, reg->sizeIdx);
+   mapHeight = dmtxGetSymbolAttribute(DmtxSymAttribDataRegionRows, reg->sizeIdx);
 
    weightFactor = 2 * (mapHeight + mapWidth + 2);
    assert(weightFactor > 0);
@@ -638,11 +760,10 @@ PopulateArrayFromMatrix(DmtxMessage *message, DmtxImage *image, DmtxRegion *regi
          xOrigin = xRegionCount * (mapWidth + 2) + 1;
 
          memset(tally, 0x00, 24 * 24 * sizeof(int));
-         /* XXX see if there's a clean way to remove decode from this call */
-         TallyModuleJumps(image, region, tally, xOrigin, yOrigin, mapWidth, mapHeight, DmtxDirUp);
-         TallyModuleJumps(image, region, tally, xOrigin, yOrigin, mapWidth, mapHeight, DmtxDirLeft);
-         TallyModuleJumps(image, region, tally, xOrigin, yOrigin, mapWidth, mapHeight, DmtxDirDown);
-         TallyModuleJumps(image, region, tally, xOrigin, yOrigin, mapWidth, mapHeight, DmtxDirRight);
+         TallyModuleJumps(img, reg, tally, xOrigin, yOrigin, mapWidth, mapHeight, DmtxDirUp);
+         TallyModuleJumps(img, reg, tally, xOrigin, yOrigin, mapWidth, mapHeight, DmtxDirLeft);
+         TallyModuleJumps(img, reg, tally, xOrigin, yOrigin, mapWidth, mapHeight, DmtxDirDown);
+         TallyModuleJumps(img, reg, tally, xOrigin, yOrigin, mapWidth, mapHeight, DmtxDirRight);
 
          /* Decide module status based on final tallies */
          for(mapRow = 0; mapRow < mapHeight; mapRow++) {
@@ -653,12 +774,12 @@ PopulateArrayFromMatrix(DmtxMessage *message, DmtxImage *image, DmtxRegion *regi
                colTmp = (xRegionCount * mapWidth) + mapCol;
                idx = (rowTmp * xRegionTotal * mapWidth) + colTmp;
 
-               if(tally[mapRow][mapCol]/(double)weightFactor > 0.5)
-                  message->array[idx] = DMTX_MODULE_ON_RGB;
+               if(tally[mapRow][mapCol]/(double)weightFactor >= 0.5)
+                  msg->array[idx] = DMTX_MODULE_ON_RGB;
                else
-                  message->array[idx] = DMTX_MODULE_OFF;
+                  msg->array[idx] = DMTX_MODULE_OFF;
 
-               message->array[idx] |= DMTX_MODULE_ASSIGNED;
+               msg->array[idx] |= DMTX_MODULE_ASSIGNED;
             }
          }
       }
@@ -668,19 +789,19 @@ PopulateArrayFromMatrix(DmtxMessage *message, DmtxImage *image, DmtxRegion *regi
 }
 
 /**
- * @brief  XXX
- * @param  image
- * @param  region
+ * @brief  Increment counters used to determine module values
+ * @param  img
+ * @param  reg
  * @param  tally
  * @param  xOrigin
  * @param  yOrigin
  * @param  mapWidth
  * @param  mapHeight
- * @param  direction
+ * @param  dir
  * @return void
  */
 static void
-TallyModuleJumps(DmtxImage *image, DmtxRegion *region, int tally[][24], int xOrigin, int yOrigin, int mapWidth, int mapHeight, int direction)
+TallyModuleJumps(DmtxImage *img, DmtxRegion *reg, int tally[][24], int xOrigin, int yOrigin, int mapWidth, int mapHeight, DmtxDirection dir)
 {
    int extent, weight;
    int travelStep;
@@ -689,20 +810,20 @@ TallyModuleJumps(DmtxImage *image, DmtxRegion *region, int tally[][24], int xOri
    int lineStart, lineStop;
    int travelStart, travelStop;
    int *line, *travel;
-   double jumpThreshold;
-   DmtxColor3 color;
+   int jumpThreshold;
+   int darkOnLight;
+   int color;
    int statusPrev, statusModule;
-   double tPrev, tModule;
+   int tPrev, tModule;
 
-   assert(direction == DmtxDirUp || direction == DmtxDirLeft ||
-         direction == DmtxDirDown || direction == DmtxDirRight);
+   assert(dir == DmtxDirUp || dir == DmtxDirLeft || dir == DmtxDirDown || dir == DmtxDirRight);
 
-   travelStep = (direction == DmtxDirUp || direction == DmtxDirRight) ? 1 : -1;
+   travelStep = (dir == DmtxDirUp || dir == DmtxDirRight) ? 1 : -1;
 
    /* Abstract row and column progress using pointers to allow grid
       traversal in all 4 directions using same logic */
 
-   if(direction & DmtxDirHorizontal) {
+   if((dir & DmtxDirHorizontal) != 0x00) {
       line = &symbolRow;
       travel = &symbolCol;
       extent = mapWidth;
@@ -712,7 +833,7 @@ TallyModuleJumps(DmtxImage *image, DmtxRegion *region, int tally[][24], int xOri
       travelStop = (travelStep == 1) ? xOrigin + mapWidth : xOrigin - 1;
    }
    else {
-      assert(direction & DmtxDirVertical);
+      assert(dir & DmtxDirVertical);
       line = &symbolCol;
       travel = &symbolRow;
       extent = mapHeight;
@@ -722,9 +843,11 @@ TallyModuleJumps(DmtxImage *image, DmtxRegion *region, int tally[][24], int xOri
       travelStop = (travelStep == 1) ? yOrigin + mapHeight : yOrigin - 1;
    }
 
-   jumpThreshold = 0.3 * (region->gradient.tMax - region->gradient.tMin);
 
-   assert(jumpThreshold > 0);
+   darkOnLight = (int)(reg->offColor > reg->onColor);
+   jumpThreshold = abs((int)(0.4 * (reg->offColor - reg->onColor) + 0.5));
+
+   assert(jumpThreshold >= 0);
 
    for(*line = lineStart; *line < lineStop; (*line)++) {
 
@@ -732,10 +855,10 @@ TallyModuleJumps(DmtxImage *image, DmtxRegion *region, int tally[][24], int xOri
          decide status based on predictable barcode border pattern */
 
       *travel = travelStart;
-      color = ReadModuleColor(image, region, symbolRow, symbolCol, region->sizeIdx);
-      tModule = dmtxDistanceAlongRay3(&(region->gradient.ray), &color);
+      color = ReadModuleColor(img, reg, symbolRow, symbolCol, reg->sizeIdx);
+      tModule = (darkOnLight) ? reg->offColor - color : color - reg->offColor;
 
-      statusModule = (travelStep == 1 || !(*line & 0x01)) ? DMTX_MODULE_ON_RGB : DMTX_MODULE_OFF;
+      statusModule = (travelStep == 1 || (*line & 0x01) == 0) ? DMTX_MODULE_ON_RGB : DMTX_MODULE_OFF;
 
       weight = extent;
 
@@ -747,8 +870,8 @@ TallyModuleJumps(DmtxImage *image, DmtxRegion *region, int tally[][24], int xOri
          /* For normal data-bearing modules capture color and decide
             module status based on comparison to previous "known" module */
 
-         color = ReadModuleColor(image, region, symbolRow, symbolCol, region->sizeIdx);
-         tModule = dmtxDistanceAlongRay3(&(region->gradient.ray), &color);
+         color = ReadModuleColor(img, reg, symbolRow, symbolCol, reg->sizeIdx);
+         tModule = (darkOnLight) ? reg->offColor - color : color - reg->offColor;
 
          if(statusPrev == DMTX_MODULE_ON_RGB) {
             if(tModule < tPrev - jumpThreshold)
@@ -769,10 +892,6 @@ TallyModuleJumps(DmtxImage *image, DmtxRegion *region, int tally[][24], int xOri
 
          if(statusModule == DMTX_MODULE_ON_RGB)
             tally[mapRow][mapCol] += (2 * weight);
-/*       else if(statusModule == DMTX_MODULE_UNSURE)
-            tally[mapRow][mapCol] += weight; */
-
-/*       debug[mapRow][mapCol] = tModule - tPrev; */
 
          weight--;
       }
@@ -782,53 +901,57 @@ TallyModuleJumps(DmtxImage *image, DmtxRegion *region, int tally[][24], int xOri
 }
 
 /**
- * @brief  XXX
- * @param  message
- * @param  image
- * @param  region
+ * @brief  Populate array with codeword values based on module colors
+ * @param  msg
+ * @param  img
+ * @param  reg
  * @return DMTX_SUCCESS | DMTX_FAILURE
  */
 static int
-PopulateArrayFromMosaic(DmtxMessage *message, DmtxImage *image, DmtxRegion *region)
+PopulateArrayFromMosaic(DmtxMessage *msg, DmtxImage *img, DmtxRegion *reg)
 {
    int col, row, rowTmp;
    int symbolRow, symbolCol;
    int dataRegionRows, dataRegionCols;
-   DmtxColor3 color;
+   int color;
 
-   dataRegionRows = dmtxGetSymbolAttribute(DmtxSymAttribDataRegionRows, region->sizeIdx);
-   dataRegionCols = dmtxGetSymbolAttribute(DmtxSymAttribDataRegionCols, region->sizeIdx);
+   dataRegionRows = dmtxGetSymbolAttribute(DmtxSymAttribDataRegionRows, reg->sizeIdx);
+   dataRegionCols = dmtxGetSymbolAttribute(DmtxSymAttribDataRegionCols, reg->sizeIdx);
 
-   memset(message->array, 0x00, message->arraySize);
+   memset(msg->array, 0x00, msg->arraySize);
 
-   for(row = 0; row < region->mappingRows; row++) {
+   for(row = 0; row < reg->mappingRows; row++) {
 
       /* Transform mapping row to symbol row (Swap because the array's
          origin is top-left and everything else is bottom-left) */
-      rowTmp = region->mappingRows - row - 1;
+      rowTmp = reg->mappingRows - row - 1;
       symbolRow = rowTmp + 2 * (rowTmp / dataRegionRows) + 1;
 
-      for(col = 0; col < region->mappingCols; col++) {
+      for(col = 0; col < reg->mappingCols; col++) {
 
          /* Transform mapping col to symbol col */
          symbolCol = col + 2 * (col / dataRegionCols) + 1;
 
-         color = ReadModuleColor(image, region, symbolRow, symbolCol, region->sizeIdx);
+/* to fix this function, add rColor, gColor, bColor, and change ReadModuleColor() to accept plane as a parameter */
+         color = ReadModuleColor(img, reg, symbolRow, symbolCol, reg->sizeIdx);
 
          /* Value has been assigned, but not visited */
-         if(color.R < 50)
-            message->array[row*region->mappingCols+col] |= DMTX_MODULE_ON_RED;
-         if(color.G < 50)
-            message->array[row*region->mappingCols+col] |= DMTX_MODULE_ON_GREEN;
-         if(color.B < 50)
-            message->array[row*region->mappingCols+col] |= DMTX_MODULE_ON_BLUE;
+/*       if(color.R < 50) this is broken for the moment */
+         if(color < 50)
+            msg->array[row*reg->mappingCols+col] |= DMTX_MODULE_ON_RED;
+/*       if(color.G < 50) this is broken for the moment */
+         if(color < 50)
+            msg->array[row*reg->mappingCols+col] |= DMTX_MODULE_ON_GREEN;
+/*       if(color.B < 50) this is broken for the moment */
+         if(color < 50)
+            msg->array[row*reg->mappingCols+col] |= DMTX_MODULE_ON_BLUE;
 
-         message->array[row*region->mappingCols+col] |= DMTX_MODULE_ASSIGNED;
+         msg->array[row*reg->mappingCols+col] |= DMTX_MODULE_ASSIGNED;
       }
    }
 
    /* Ideal barcode drawn in lower-right (final) window pane */
-/* CALLBACK_DECODE_FUNC2(finalCallback, dec, dec, region); */
+/* CALLBACK_DECODE_FUNC2(finalCallback, dec, dec, reg); */
 
    return DMTX_SUCCESS;
 }
