@@ -20,7 +20,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mike@dragonflylogic.com
 */
 
-/* $Id: dmtxencode.c 185 2008-06-17 20:07:47Z mblaughton $ */
+/* $Id: dmtxencode.c 514 2008-11-19 17:10:44Z mblaughton $ */
 
 /**
  * @file dmtxencode.c
@@ -66,7 +66,7 @@ Contact: mike@dragonflylogic.com
 #define DMTX_UNLATCH_IMPLICIT            1
 
 /**
- * @brief  XXX
+ * @brief  Initialize encode struct with default values
  * @return Initialized DmtxEncode struct
  */
 extern DmtxEncode
@@ -76,25 +76,21 @@ dmtxEncodeStructInit(void)
 
    memset(&enc, 0x00, sizeof(DmtxEncode));
 
-   enc.scheme = DmtxSchemeEncodeAutoBest;
+   enc.scheme = DmtxSchemeEncodeAscii;
    enc.moduleSize = 5;
    enc.marginSize = 10;
 
-   /* This can be cleaned up later */
-   enc.region.gradient.isDefined = DMTX_TRUE;
-
    /* Initialize background color to white */
-   enc.region.gradient.ray.p.R = 255.0;
+/* enc.region.gradient.ray.p.R = 255.0;
    enc.region.gradient.ray.p.G = 255.0;
-   enc.region.gradient.ray.p.B = 255.0;
+   enc.region.gradient.ray.p.B = 255.0; */
 
    /* Initialize foreground color to black */
-   enc.region.gradient.tMin = 0.0;
-   enc.region.gradient.tMax = dmtxColor3Mag(&(enc.region.gradient.ray.p));
-   enc.region.gradient.tMid = (enc.region.gradient.tMin + enc.region.gradient.tMax)/2.0;
+/* enc.region.gradient.tMin = 0.0;
+   enc.region.gradient.tMax = dmtxColor3Mag(&(enc.region.gradient.ray.p)); */
 
-   dmtxColor3Scale(&(enc.region.gradient.ray.c),
-         &(enc.region.gradient.ray.p), -1.0/enc.region.gradient.tMax);
+/* dmtxColor3Scale(&(enc.region.gradient.ray.c),
+         &(enc.region.gradient.ray.p), -1.0/enc.region.gradient.tMax); */
 
    dmtxMatrix3Identity(enc.xfrm);
 
@@ -102,7 +98,7 @@ dmtxEncodeStructInit(void)
 }
 
 /**
- * @brief  XXX
+ * @brief  Deinitialize encode struct
  * @param  enc
  * @return void
  */
@@ -119,7 +115,7 @@ dmtxEncodeStructDeInit(DmtxEncode *enc)
 }
 
 /**
- * @brief  XXX
+ * @brief  Convert message into Data Matrix image
  * @param  enc
  * @param  inputSize
  * @param  inputString
@@ -133,17 +129,14 @@ dmtxEncodeDataMatrix(DmtxEncode *enc, int inputSize, unsigned char *inputString,
    int sizeIdx;
    unsigned char buf[4096];
 
-   /* XXX must fix ... will need to handle sizeIdx requests here because it is
-      needed by Encode...() (for triplet termination) */
-
    /* Encode input string into data codewords */
    sizeIdx = sizeIdxRequest;
    dataWordCount = EncodeDataCodewords(buf, inputString, inputSize, enc->scheme, &sizeIdx);
    if(dataWordCount <= 0)
-      exit(1);
+      return(DMTX_FAILURE);
 
    /* EncodeDataCodewords() should have updated any auto sizeIdx to a real one */
-   assert(sizeIdx != DMTX_SYMBOL_SQUARE_AUTO && sizeIdx != DMTX_SYMBOL_RECT_AUTO);
+   assert(sizeIdx != DmtxSymbolSquareAuto && sizeIdx != DmtxSymbolRectAuto);
 
    /* Add pad characters to match a standard symbol size (whether smallest or requested) */
    AddPadChars(buf, &dataWordCount, dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, sizeIdx));
@@ -168,14 +161,13 @@ dmtxEncodeDataMatrix(DmtxEncode *enc, int inputSize, unsigned char *inputString,
    ModulePlacementEcc200(enc->message->array, enc->message->code, enc->region.sizeIdx, DMTX_MODULE_ON_RGB);
 
    /* Allocate memory for the image to be generated */
-   /* XXX image = DmtxImageMalloc(width, height); */
    enc->image = dmtxImageMalloc(
          2 * enc->marginSize + (enc->region.symbolCols * enc->moduleSize),
          2 * enc->marginSize + (enc->region.symbolRows * enc->moduleSize));
 
    if(enc->image == NULL) {
       perror("image malloc error");
-      exit(2);
+      return DMTX_FAILURE;
    }
 
    /* Insert finder and aligment pattern modules */
@@ -185,7 +177,7 @@ dmtxEncodeDataMatrix(DmtxEncode *enc, int inputSize, unsigned char *inputString,
 }
 
 /**
- * @brief  XXX
+ * @brief  Convert message into Data Mosaic image
  * @param  enc
  * @param  inputSize
  * @param  inputString
@@ -223,7 +215,7 @@ dmtxEncodeDataMosaic(DmtxEncode *enc, int inputSize, unsigned char *inputString,
    /* XXX buf can be changed here to use all 3 buffers' length */
    dataWordCount = EncodeDataCodewords(buf[0], inputString, inputSize, enc->scheme, &sizeIdx);
    if(dataWordCount <= 0)
-      exit(1);
+      return DMTX_FAILURE;
 
    /* Use 1/3 (ceiling) of inputSize establish input size target */
    tmpInputSize = (inputSize + 2) / 3;
@@ -233,12 +225,14 @@ dmtxEncodeDataMosaic(DmtxEncode *enc, int inputSize, unsigned char *inputString,
    /* XXX clean up above lines later for corner cases */
 
    /* Use 1/3 (floor) of dataWordCount establish first symbol size attempt */
-   splitSizeIdxFirst = FindCorrectBarcodeSize(tmpInputSize, sizeIdxRequest);
+   splitSizeIdxFirst = FindCorrectSymbolSize(tmpInputSize, sizeIdxRequest);
+   if(splitSizeIdxFirst == -1)
+      return DMTX_FAILURE;
 
    /* Set the last possible symbol size for this symbol shape or specific size request */
-   if(sizeIdxRequest == DMTX_SYMBOL_SQUARE_AUTO)
+   if(sizeIdxRequest == DmtxSymbolSquareAuto)
       splitSizeIdxLast = DMTX_SYMBOL_SQUARE_COUNT - 1;
-   else if(sizeIdxRequest == DMTX_SYMBOL_RECT_AUTO)
+   else if(sizeIdxRequest == DmtxSymbolRectAuto)
       splitSizeIdxLast = DMTX_SYMBOL_SQUARE_COUNT + DMTX_SYMBOL_RECT_COUNT - 1;
    else
       splitSizeIdxLast = splitSizeIdxFirst;
@@ -325,7 +319,7 @@ dmtxEncodeDataMosaic(DmtxEncode *enc, int inputSize, unsigned char *inputString,
 }
 
 /**
- * @brief  XXX
+ * @brief  Convert input into message using specific encodation scheme
  * @param  buf
  * @param  inputString
  * @param  inputSize
@@ -363,13 +357,15 @@ EncodeDataCodewords(unsigned char *buf, unsigned char *inputString,
       needed by Encode...() for triplet termination */
 
    /* parameter sizeIdx is requested value, returned sizeIdx is decision */
-   *sizeIdx = FindCorrectBarcodeSize(dataWordCount, *sizeIdx);
+   *sizeIdx = FindCorrectSymbolSize(dataWordCount, *sizeIdx);
+   if(*sizeIdx == -1)
+      return 0;
 
    return dataWordCount;
 }
 
 /**
- * @brief  XXX
+ * @brief  Add necessary padding codewords to message
  * @param  buf
  * @param  bufSize
  * @param  paddedSize
@@ -390,7 +386,7 @@ AddPadChars(unsigned char *buf,  int *bufSize, int paddedSize)
 }
 
 /**
- * @brief  XXX
+ * @brief  Randomize 253 state
  * @param  codewordValue
  * @param  codewordPosition
  * @return Randomized value
@@ -408,7 +404,7 @@ Randomize253State(unsigned char codewordValue, int codewordPosition)
 }
 
 /**
- * @brief  XXX
+ * @brief  Randomize 255 state
  * @param  codewordValue
  * @param  codewordPosition
  * @return Randomized value
@@ -426,7 +422,7 @@ Randomize255State(unsigned char codewordValue, int codewordPosition)
 }
 
 /**
- * @brief  XXX
+ * @brief  Write encoded message to image
  * @param  enc
  * @return void
  */
@@ -436,10 +432,11 @@ PrintPattern(DmtxEncode *enc)
    int i, j;
    int symbolRow, symbolCol;
    int pixelRow, pixelCol;
+   int moduleStatus;
    double sxy, txy;
+   DmtxRgb rgb;
    DmtxMatrix3 m1, m2;
    DmtxVector2 vIn, vOut;
-   int moduleStatus;
 
    txy = enc->marginSize;
    sxy = 1.0/enc->moduleSize;
@@ -456,7 +453,7 @@ PrintPattern(DmtxEncode *enc)
       IMPORTANT: DmtxImage is stored with its origin at bottom-right
       (unlike common image file formats) to preserve "right-handed" 2D space */
 
-   memset(enc->image->pxl, 0xff, enc->image->width * enc->image->height * sizeof(DmtxRgb));
+   memset(enc->image->pxl, 0xff, dmtxImageGetProp(enc->image, DmtxPropArea) * sizeof(DmtxRgb));
 
    for(symbolRow = 0; symbolRow < enc->region.symbolRows; symbolRow++) {
       for(symbolCol = 0; symbolCol < enc->region.symbolCols; symbolCol++) {
@@ -473,12 +470,10 @@ PrintPattern(DmtxEncode *enc)
 
          for(i = pixelRow; i < pixelRow + enc->moduleSize; i++) {
             for(j = pixelCol; j < pixelCol + enc->moduleSize; j++) {
-/*             enc->image->pxl[i * enc->image->width + j].R = (moduleStatus & DMTX_MODULE_ON_RED) ? 0 : 255;
-               enc->image->pxl[i * enc->image->width + j].G = (moduleStatus & DMTX_MODULE_ON_GREEN) ? 0 : 255;
-               enc->image->pxl[i * enc->image->width + j].B = (moduleStatus & DMTX_MODULE_ON_BLUE) ? 0 : 255; */
-               enc->image->pxl[i * enc->image->width + j][0] = (moduleStatus & DMTX_MODULE_ON_RED) ? 0 : 255;
-               enc->image->pxl[i * enc->image->width + j][1] = (moduleStatus & DMTX_MODULE_ON_GREEN) ? 0 : 255;
-               enc->image->pxl[i * enc->image->width + j][2] = (moduleStatus & DMTX_MODULE_ON_BLUE) ? 0 : 255;
+               rgb[0] = ((moduleStatus & DMTX_MODULE_ON_RED) != 0x00) ? 0 : 255;
+               rgb[1] = ((moduleStatus & DMTX_MODULE_ON_GREEN) != 0x00) ? 0 : 255;
+               rgb[2] = ((moduleStatus & DMTX_MODULE_ON_BLUE) != 0x00) ? 0 : 255;
+               dmtxImageSetRgb(enc->image, j, i, rgb);
             }
          }
 
@@ -487,7 +482,7 @@ PrintPattern(DmtxEncode *enc)
 }
 
 /**
- * @brief  XXX
+ * @brief  Initialize encoding channel
  * @param  channel
  * @param  codewords
  * @param  length
@@ -504,7 +499,7 @@ InitChannel(DmtxChannel *channel, unsigned char *codewords, int length)
 }
 
 /**
- * @brief  XXX
+ * @brief  Encode message using single encodation scheme
  * @param  buf
  * @param  codewords
  * @param  length
@@ -517,8 +512,6 @@ EncodeSingleScheme(unsigned char *buf, unsigned char *codewords, int length, Dmt
    int size;
    DmtxChannel channel;
 
-   /* XXX function needs to change return size */
-
    InitChannel(&channel, codewords, length);
 
    while(channel.inputPtr < channel.inputStop) {
@@ -526,7 +519,7 @@ EncodeSingleScheme(unsigned char *buf, unsigned char *codewords, int length, Dmt
 
       /* DumpChannel(&channel); */
 
-      if(channel.invalid) {
+      if(channel.invalid != 0) {
          fprintf(stderr, "Character \"%c\" not supported by requested encodation scheme\n\n", *channel.inputPtr);
          return 0;
       }
@@ -540,7 +533,7 @@ EncodeSingleScheme(unsigned char *buf, unsigned char *codewords, int length, Dmt
 }
 
 /**
- * @brief  XXX
+ * @brief  Encode message using best possible encodation (combine schemes)
  * @param  buf
  * @param  codewords
  * @param  length
@@ -554,8 +547,6 @@ EncodeAutoBest(unsigned char *buf, unsigned char *codewords, int length)
    DmtxChannelGroup optimal, best;
    DmtxChannel *channel, *winner;
 
-   /* XXX function needs to change return size */
-
    /* Intialize optimizing channels and encode first codeword from default ASCII */
    for(targetScheme = DmtxSchemeEncodeAscii; targetScheme <= DmtxSchemeEncodeBase256; targetScheme++) {
       channel = &(optimal.channel[targetScheme]);
@@ -600,7 +591,7 @@ EncodeAutoBest(unsigned char *buf, unsigned char *codewords, int length)
 }
 
 /**
- * @brief  XXX
+ * @brief  Determine current best channel in encoding process
  * @param  group
  * @param  targetScheme
  * @return Winning channel
@@ -618,7 +609,7 @@ FindBestChannel(DmtxChannelGroup group, DmtxSchemeEncode targetScheme)
 
       /* If from channel doesn't hold valid data because it couldn't
          represent the previous value then skip it */
-      if(channel->invalid)
+      if(channel->invalid != 0)
          continue;
 
       /* If channel has already processed all of its input values then it
@@ -629,13 +620,13 @@ FindBestChannel(DmtxChannelGroup group, DmtxSchemeEncode targetScheme)
       EncodeNextWord(channel, targetScheme);
 
       /* If channel scheme can't represent next word then stop for this channel */
-      if(channel->invalid & DMTX_CHANNEL_UNSUPPORTED_CHAR) {
+      if((channel->invalid & DMTX_CHANNEL_UNSUPPORTED_CHAR) != 0) {
          winner = channel;
          break;
       }
 
       /* If channel scheme was unable to unlatch here then skip */
-      if(channel->invalid & DMTX_CHANNEL_CANNOT_UNLATCH)
+      if((channel->invalid & DMTX_CHANNEL_CANNOT_UNLATCH) != 0)
          continue;
 
       if(winner == NULL || channel->currentLength < winner->currentLength)
@@ -648,7 +639,7 @@ FindBestChannel(DmtxChannelGroup group, DmtxSchemeEncode targetScheme)
 }
 
 /**
- * @brief  XXX
+ * @brief  Encode next codeword using requested encodation scheme
  * @param  channel
  * @param  targetScheme
  * @return void
@@ -659,7 +650,7 @@ EncodeNextWord(DmtxChannel *channel, DmtxSchemeEncode targetScheme)
    /* Change to new encodation scheme if necessary */
    if(channel->encScheme != targetScheme) {
       ChangeEncScheme(channel, targetScheme, DMTX_UNLATCH_EXPLICIT);
-      if(channel->invalid)
+      if(channel->invalid != 0)
          return;
    }
 
@@ -697,7 +688,7 @@ EncodeNextWord(DmtxChannel *channel, DmtxSchemeEncode targetScheme)
 }
 
 /**
- * @brief  XXX
+ * @brief  Encode value using ASCII encodation (standard or extended)
  * @param  channel
  * @return void
  */
@@ -746,10 +737,12 @@ EncodeAsciiCodeword(DmtxChannel *channel)
       as separate channels. */
 
    /* 2nd digit char in a row - overwrite first digit word with combined value */
-   if(isdigit(inputValue) && channel->currentLength >= channel->firstCodeWord + 24) {
+   if(isdigit(inputValue) && channel->currentLength >= channel->firstCodeWord + 12) {
       prevIndex = (channel->currentLength - 12)/12;
       prevValue = channel->encodedWords[prevIndex] - 1;
-      prevPrevValue = channel->encodedWords[prevIndex-1];
+
+      prevPrevValue = (prevIndex > channel->firstCodeWord/12) ?
+            channel->encodedWords[prevIndex-1] : 0;
 
       if(prevPrevValue != 235 && isdigit(prevValue)) {
          channel->encodedWords[prevIndex] = 10 * (prevValue - '0') + (inputValue - '0') + 130;
@@ -775,7 +768,7 @@ EncodeAsciiCodeword(DmtxChannel *channel)
 }
 
 /**
- * @brief  XXX
+ * @brief  Encode value using C40, Text, or X12 encodation
  * @param  channel
  * @return void
  */
@@ -890,7 +883,7 @@ EncodeTripletCodeword(DmtxChannel *channel)
 }
 
 /**
- * @brief  XXX
+ * @brief  Encode value using EDIFACT encodation
  * @param  channel
  * @return void
  */
@@ -917,7 +910,7 @@ EncodeEdifactCodeword(DmtxChannel *channel)
 }
 
 /**
- * @brief  XXX
+ * @brief  Encode value using Base 256 encodation
  * @param  channel
  * @return void
  */
@@ -961,7 +954,7 @@ EncodeBase256Codeword(DmtxChannel *channel)
    /* newDataLength does not include header bytes */
    assert(newDataLength > 0 && newDataLength <= 1555);
 
-   /* One time shift of codewords when passing the 250 byte size threshhold */
+   /* One time shift of codewords when passing the 250 byte size threshold */
    if(newDataLength == 250) {
       for(i = channel->currentLength/12 - 1; i > channel->firstCodeWord/12; i--) {
          valueTmp = UnRandomize255State(channel->encodedWords[i], i+1);
@@ -985,7 +978,7 @@ EncodeBase256Codeword(DmtxChannel *channel)
 }
 
 /**
- * @brief  XXX
+ * @brief  Change from one encodation scheme to another
  * @param  channel
  * @param  targetScheme
  * @param  unlatchType
@@ -1010,7 +1003,7 @@ ChangeEncScheme(DmtxChannel *channel, DmtxSchemeEncode targetScheme, int unlatch
       case DmtxSchemeEncodeX12:
 
          /* Can't unlatch unless currently at a byte boundary */
-         if(channel->currentLength % 12) {
+         if((channel->currentLength % 12) != 0) {
             channel->invalid = DMTX_CHANNEL_CANNOT_UNLATCH;
             return;
          }
@@ -1113,7 +1106,7 @@ ChangeEncScheme(DmtxChannel *channel, DmtxSchemeEncode targetScheme, int unlatch
 }
 
 /**
- * @brief  XXX
+ * @brief  Push codeword onto channel and increment length
  * @param  channel
  * @param  codeword
  * @return void
@@ -1128,8 +1121,8 @@ PushInputWord(DmtxChannel *channel, unsigned char codeword)
    /* XXX should this assertion actually be a legit runtime test? */
    assert(channel->encodedLength/12 <= 3*1558); /* increased for Mosaic */
 
-   /* XXX this is currently pretty ugly, but I have other fish to fry at
-      the moment.  What is required is to go through and decide on a
+   /* XXX this is currently pretty ugly, but can wait until the
+      rewrite. What is required is to go through and decide on a
       consistent approach (probably that all encodation schemes use
       currentLength except for triplet-based schemes which use
       currentLength and encodedLength).  All encodation schemes should
@@ -1195,7 +1188,7 @@ PushInputWord(DmtxChannel *channel, unsigned char codeword)
 }
 
 /**
- * @brief  XXX
+ * @brief  Push triplet codeword onto channel
  * @param  channel
  * @param  triplet
  * @return void
@@ -1211,7 +1204,7 @@ PushTriplet(DmtxChannel *channel, DmtxTriplet *triplet)
 }
 
 /**
- * @brief  XXX
+ * @brief  Increment encoding progress tracking variables
  * @param  channel
  * @param  encodedUnits
  * @return void
@@ -1251,7 +1244,7 @@ IncrementProgress(DmtxChannel *channel, int encodedUnits)
 }
 
 /**
- * @brief  XXX
+ * @brief  Special end-of-symbol encoding for triplet-based schemes
  * @param  channel
  * @param  triplet
  * @param  tripletCount
@@ -1302,9 +1295,10 @@ ProcessEndOfSymbolTriplet(DmtxChannel *channel, DmtxTriplet *triplet, int triple
 
    /* Find minimum symbol size big enough to accomodate remaining codewords */
    currentByte = channel->currentLength/12;
-/* XXX this is broken -- what if someone asks for DMTX_SYMBOL_RECT_AUTO or a specific sizeIdx? */
-   sizeIdx = FindCorrectBarcodeSize(currentByte + ((inputCount == 3) ? 2 : inputCount),
-         DMTX_SYMBOL_SQUARE_AUTO);
+/* XXX this is broken -- what if someone asks for DmtxSymbolRectAuto or a specific sizeIdx? */
+   sizeIdx = FindCorrectSymbolSize(currentByte + ((inputCount == 3) ? 2 : inputCount),
+         DmtxSymbolSquareAuto);
+   /* XXX test for sizeIdx == -1 here */
    remainingCodewords = dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, sizeIdx) - currentByte;
 
    /* XXX the big problem with all of these special cases is what if one of
@@ -1370,7 +1364,7 @@ ProcessEndOfSymbolTriplet(DmtxChannel *channel, DmtxTriplet *triplet, int triple
 }
 
 /**
- * @brief  XXX
+ * @brief  Determine if end-of-symbol condition is met for EDIFACT-based schemes
  * @param  channel
  * @return void
  */
@@ -1415,10 +1409,11 @@ TestForEndOfSymbolEdifact(DmtxChannel *channel)
       return;
 
    /* Find minimum symbol size big enough to accomodate remaining codewords */
-   /* XXX broken -- what if someone asks for DMTX_SYMBOL_RECT_AUTO or specific sizeIdx? */
+   /* XXX broken -- what if someone asks for DmtxSymbolRectAuto or specific sizeIdx? */
 
    currentByte = channel->currentLength/12;
-   sizeIdx = FindCorrectBarcodeSize(currentByte, DMTX_SYMBOL_SQUARE_AUTO);
+   sizeIdx = FindCorrectSymbolSize(currentByte, DmtxSymbolSquareAuto);
+   /* XXX test for sizeIdx == -1 here */
    symbolCodewords = dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, sizeIdx) - currentByte;
 
    /* Test for special case condition */
@@ -1447,7 +1442,7 @@ TestForEndOfSymbolEdifact(DmtxChannel *channel)
 }
 
 /**
- * @brief  XXX
+ * @brief  Convert 3 input values into 2 codewords for triplet-based schemes
  * @param  outputWords
  * @param  inputWord
  * @param  encScheme
@@ -1538,7 +1533,7 @@ GetC40TextX12Words(int *outputWords, int inputWord, DmtxSchemeEncode encScheme)
 }
 
 /**
- * @brief  XXX
+ * @brief  Convert 2 codewords into 3 values for triplet-based schemes
  * @param  cw1
  * @param  cw2
  * @return Triplet data
@@ -1560,7 +1555,7 @@ GetTripletValues(unsigned char cw1, unsigned char cw2)
 }
 
 /**
- * @brief  XXX
+ * @brief  Convert 3 codewords into 4 values for quadrulplet-based schemes
  * @param  cw1
  * @param  cw2
  * @param  cw3
@@ -1582,7 +1577,7 @@ GetQuadrupletValues(unsigned char cw1, unsigned char cw2, unsigned char cw3)
 }
 
 /**
- * @brief  XXX
+ * @brief  Write channel contents to standard output
  * @param  channel
  * @return void
  */
@@ -1608,7 +1603,7 @@ DumpChannel(DmtxChannel *channel)
 */
 
 /**
- * @brief  XXX
+ * @brief  Write all channels' contents to standard output
  * @param  group
  * @param  encTarget
  * @return void
