@@ -19,7 +19,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mblaughton@users.sourceforge.net
 */
 
-/* $Id: callback.h 162 2008-06-09 19:32:23Z mblaughton $ */
+/* $Id: callback.h 455 2008-10-22 17:39:04Z mblaughton $ */
 
 #ifndef __CALLBACK_H__
 #define __CALLBACK_H__
@@ -27,7 +27,7 @@ Contact: mblaughton@users.sourceforge.net
 void BuildMatrixCallback2(DmtxRegion *region);
 void BuildMatrixCallback3(DmtxMatrix3 region);
 void BuildMatrixCallback4(DmtxMatrix3 region);
-void PlotPointCallback(DmtxVector2 point, int colorInt, int paneNbr, int dispType);
+void PlotPointCallback(DmtxPixelLoc loc, int colorInt, int paneNbr, int dispType);
 void XfrmPlotPointCallback(DmtxVector2 point, DmtxMatrix3 xfrm, int paneNbr, int dispType);
 void FinalCallback(DmtxDecode *decode, DmtxRegion *region);
 void PlotModuleCallback(DmtxDecode *info, DmtxRegion *region, int row, int col, DmtxColor3 color);
