@@ -19,12 +19,13 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mblaughton@users.sourceforge.net
 */
 
-/* $Id: gltest.h 162 2008-06-09 19:32:23Z mblaughton $ */
+/* $Id: gltest.h 416 2008-08-29 18:35:57Z mblaughton $ */
 
 #ifndef __SCANDEMO_H__
 #define __SCANDEMO_H__
 
-#include <GL/gl.h>
+#include "SDL/SDL.h"
+#include "SDL/SDL_opengl.h"
 #include "dmtx.h"
 
 #define max(X,Y) (X > Y) ? X : Y
