@@ -19,7 +19,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mblaughton@users.sourceforge.net
 */
 
-/* $Id: image.h 162 2008-06-09 19:32:23Z mblaughton $ */
+/* $Id: image.h 404 2008-08-14 21:05:32Z mblaughton $ */
 
 #ifndef __IMAGE_H__
 #define __IMAGE_H__
@@ -36,10 +36,10 @@ typedef enum {
    ColorYellow = 0x01 << 4
 } ColorEnum;
 
-void captureImage(DmtxImage *image);
-int loadTextureImage(DmtxImage **image);
+void captureImage(DmtxImage *img, DmtxImage *imgTmp);
+int loadTextureImage(DmtxImage **img);
 DmtxImage *loadPng(char *filename);
-void plotPoint(DmtxImage *image, float rowFloat, float colFloat, int targetColor);
+void plotPoint(DmtxImage *img, float rowFloat, float colFloat, int targetColor);
 int clampRGB(float color);
 
 #endif
