@@ -20,7 +20,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mike@dragonflylogic.com
 */
 
-/* $Id: dmtxwrite.c 175 2008-06-13 02:44:05Z mblaughton $ */
+/* $Id: dmtxwrite.c 503 2008-11-07 18:37:35Z mblaughton $ */
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -42,71 +42,63 @@ Contact: mike@dragonflylogic.com
 #include "dmtxwrite.h"
 #include "../common/dmtxutil.h"
 
-#define MIN(x,y) ((x < y) ? x : y)
-
 char *programName;
 
 /**
- * Main function for the dmtxwrite Data Matrix scanning utility.
- *
+ * @brief  Main function for the dmtxwrite Data Matrix scanning utility.
  * @param  argc count of arguments passed from command line
  * @param  argv list of argument passed strings from command line
- * @return      numeric success / failure exit code
+ * @return Numeric exit code
  */
 int
 main(int argc, char *argv[])
 {
    int err;
-   UserOptions options;
-   DmtxEncode encode;
+   UserOptions opt;
+   DmtxEncode enc;
    unsigned char codeBuffer[DMTXWRITE_BUFFER_SIZE];
    int codeBufferSize = sizeof codeBuffer;
 
-   InitUserOptions(&options);
+   InitUserOptions(&opt);
 
    /* Create and initialize libdmtx structures */
-   encode = dmtxEncodeStructInit();
+   enc = dmtxEncodeStructInit();
 
    /* Process user options */
-   err = HandleArgs(&options, &argc, &argv, &encode);
-   if(err)
+   err = HandleArgs(&opt, &argc, &argv, &enc);
+   if(err != DMTX_SUCCESS)
       ShowUsage(err);
 
    /* Read input data into buffer */
-   ReadData(&options, &codeBufferSize, codeBuffer);
+   ReadData(&opt, &codeBufferSize, codeBuffer);
 
    /* Create barcode image */
-   if(options.mosaic)
-      dmtxEncodeDataMosaic(&encode, codeBufferSize, codeBuffer, options.sizeIdx);
+   if(opt.mosaic)
+      err = dmtxEncodeDataMosaic(&enc, codeBufferSize, codeBuffer, opt.sizeIdx);
    else
-      dmtxEncodeDataMatrix(&encode, codeBufferSize, codeBuffer, options.sizeIdx);
+      err = dmtxEncodeDataMatrix(&enc, codeBufferSize, codeBuffer, opt.sizeIdx);
+
+   if(err == DMTX_FAILURE)
+      FatalError(1, _("Unable to encode message (possibly too large for requested size)"));
 
    /* Write barcode image to requested format */
-   switch(options.format) {
-      case 'n':
-         break; /* do nothing */
+   switch(opt.format) {
       case 'p':
-         WriteImagePng(&options, &encode);
+         WriteImagePng(&opt, &enc);
          break;
       case 'm':
-         WriteImagePnm(&options, &encode);
+         WriteImagePnm(&opt, &enc);
          break;
-   }
-
-   /* Write barcode preview if requested */
-   switch(options.preview) {
-      case 'n':
-         break; /* do nothing */
       case 'a':
-         WriteAsciiBarcode(&encode);
+         WriteAsciiBarcode(&enc);
          break;
       case 'c':
-         WriteCodewords(&encode);
+         WriteCodewords(&enc);
          break;
    }
 
    /* Clean up */
-   dmtxEncodeStructDeInit(&encode);
+   dmtxEncodeStructDeInit(&enc);
 
    exit(0);
 }
@@ -116,48 +108,38 @@ main(int argc, char *argv[])
  *
  */
 static void
-InitUserOptions(UserOptions *options)
+InitUserOptions(UserOptions *opt)
 {
-   memset(options, 0x00, sizeof(UserOptions));
+   memset(opt, 0x00, sizeof(UserOptions));
 
-/* options->color = ""; */
-/* options->bgColor = ""; */
-   options->format = 'p';
-   options->inputPath = NULL;
-   options->outputPath = "dmtxwrite.png"; /* XXX needs to have different extensions for other formats */
-   options->preview = 'n';
-   options->rotate = 0;
-   options->sizeIdx = DMTX_SYMBOL_SQUARE_AUTO;
-   options->verbose = 0;
-   options->mosaic = 0;
-   options->dpi = 0; /* default to native resolution of requested image format */
+/* opt->color = ""; */
+/* opt->bgColor = ""; */
+   opt->format = 'p';
+   opt->inputPath = NULL;    /* default stdin */
+   opt->outputPath = NULL;   /* default stdout */
+   opt->rotate = 0;
+   opt->sizeIdx = DmtxSymbolSquareAuto;
+   opt->verbose = 0;
+   opt->mosaic = 0;
+   opt->dpi = 0; /* default to native resolution of requested image format */
 }
 
 /**
- * Sets and validates user-requested options from command line arguments.
- *
- * @param options runtime options from defaults or command line
- * @param argcp   pointer to argument count
- * @param argvp   pointer to argument list
- * @return        DMTXUTIL_SUCCESS | DMTXUTIL_ERROR
+ * @brief  Set and validate user-requested options from command line arguments.
+ * @param  opt runtime options from defaults or command line
+ * @param  argcp pointer to argument count
+ * @param  argvp pointer to argument list
+ * @return DMTX_SUCCESS | DMTX_FAILURE
  */
 static int
-HandleArgs(UserOptions *options, int *argcp, char **argvp[], DmtxEncode *encode)
+HandleArgs(UserOptions *opt, int *argcp, char **argvp[], DmtxEncode *enc)
 {
    int err;
    int i;
-   int opt;
+   int optchr;
    int longIndex;
    char *ptr;
 
-   static char *symbolSizes[] = {
-         "10x10", "12x12",   "14x14",   "16x16",   "18x18",   "20x20",
-         "22x22", "24x24",   "26x26",   "32x32",   "36x36",   "40x40",
-         "44x44", "48x48",   "52x52",   "64x64",   "72x72",   "80x80",
-         "88x88", "96x96", "104x104", "120x120", "132x132", "144x144",
-          "8x18",  "8x32",   "12x26",   "12x36",   "16x36",   "16x48"
-   };
-
    struct option longOptions[] = {
          {"color",       required_argument, NULL, 'c'},
          {"bg-color",    required_argument, NULL, 'b'},
@@ -166,7 +148,6 @@ HandleArgs(UserOptions *options, int *argcp, char **argvp[], DmtxEncode *encode)
          {"encoding",    required_argument, NULL, 'e'},
          {"format",      required_argument, NULL, 'f'},
          {"output",      required_argument, NULL, 'o'},
-         {"preview",     required_argument, NULL, 'p'},
          {"rotate",      required_argument, NULL, 'r'},
          {"symbol-size", required_argument, NULL, 's'},
          {"verbose",     no_argument,       NULL, 'v'},
@@ -180,136 +161,123 @@ HandleArgs(UserOptions *options, int *argcp, char **argvp[], DmtxEncode *encode)
    programName = Basename((*argvp)[0]);
 
    /* Set default values before considering arguments */
-   encode->moduleSize = 5;
-   encode->marginSize = 10;
-   encode->scheme = DmtxSchemeEncodeAscii;
+   enc->moduleSize = 5;
+   enc->marginSize = 10;
+   enc->scheme = DmtxSchemeEncodeAscii;
 
    for(;;) {
-      opt = getopt_long(*argcp, *argvp, "c:b:d:m:e:f:o:p:r:s:vMR:V", longOptions, &longIndex);
-      if(opt == -1)
+      optchr = getopt_long(*argcp, *argvp, "c:b:d:m:e:f:o:r:s:vMR:V", longOptions, &longIndex);
+      if(optchr == -1)
          break;
 
-      switch(opt) {
+      switch(optchr) {
          case 0: /* --help */
             ShowUsage(0);
             break;
          case 'c':
-            options->color[0] = 0;
-            options->color[1] = 0;
-            options->color[2] = 0;
-            fprintf(stdout, "Option \"%c\" not implemented yet\n", opt);
+            opt->color[0] = 0;
+            opt->color[1] = 0;
+            opt->color[2] = 0;
+            fprintf(stdout, "Option \"%c\" not implemented\n", optchr);
             break;
          case 'b':
-            options->bgColor[0] = 255;
-            options->bgColor[1] = 255;
-            options->bgColor[2] = 255;
-            fprintf(stdout, "Option \"%c\" not implemented yet\n", opt);
+            opt->bgColor[0] = 255;
+            opt->bgColor[1] = 255;
+            opt->bgColor[2] = 255;
+            fprintf(stdout, "Option \"%c\" not implemented\n", optchr);
             break;
          case 'd':
-            err = StringToInt(&(encode->moduleSize), optarg, &ptr);
-            if(err != DMTXUTIL_SUCCESS || encode->moduleSize <= 0 || *ptr != '\0')
+            err = StringToInt(&(enc->moduleSize), optarg, &ptr);
+            if(err != DMTX_SUCCESS || enc->moduleSize <= 0 || *ptr != '\0')
                FatalError(1, _("Invalid module size specified \"%s\""), optarg);
             break;
          case 'm':
-            err = StringToInt(&(encode->marginSize), optarg, &ptr);
-            if(err != DMTXUTIL_SUCCESS || encode->marginSize <= 0 || *ptr != '\0')
+            err = StringToInt(&(enc->marginSize), optarg, &ptr);
+            if(err != DMTX_SUCCESS || enc->marginSize <= 0 || *ptr != '\0')
                FatalError(1, _("Invalid margin size specified \"%s\""), optarg);
             break;
          case 'e':
             if(strlen(optarg) != 1) {
                fprintf(stdout, "Invalid encodation scheme \"%s\"\n", optarg);
-               return DMTXUTIL_ERROR;
+               return DMTX_FAILURE;
             }
             switch(*optarg) {
                case 'b':
-                  encode->scheme = DmtxSchemeEncodeAutoBest;
+                  enc->scheme = DmtxSchemeEncodeAutoBest;
                   break;
                case 'f':
-                  encode->scheme = DmtxSchemeEncodeAutoFast;
-                  fprintf(stdout, "\"Fast optimized\" not implemented yet\n", opt);
-                  return DMTXUTIL_ERROR;
+                  enc->scheme = DmtxSchemeEncodeAutoFast;
+                  fprintf(stdout, "\"Fast optimized\" not implemented\n");
+                  return DMTX_FAILURE;
                   break;
                case 'a':
-                  encode->scheme = DmtxSchemeEncodeAscii;
+                  enc->scheme = DmtxSchemeEncodeAscii;
                   break;
                case 'c':
-                  encode->scheme = DmtxSchemeEncodeC40;
+                  enc->scheme = DmtxSchemeEncodeC40;
                   break;
                case 't':
-                  encode->scheme = DmtxSchemeEncodeText;
+                  enc->scheme = DmtxSchemeEncodeText;
                   break;
                case 'x':
-                  encode->scheme = DmtxSchemeEncodeX12;
+                  enc->scheme = DmtxSchemeEncodeX12;
                   break;
                case 'e':
-                  encode->scheme = DmtxSchemeEncodeEdifact;
+                  enc->scheme = DmtxSchemeEncodeEdifact;
                   break;
                case '8':
-                  encode->scheme = DmtxSchemeEncodeBase256;
+                  enc->scheme = DmtxSchemeEncodeBase256;
                   break;
                default:
                   fprintf(stdout, "Invalid encodation scheme \"%s\"\n", optarg);
-                  return DMTXUTIL_ERROR;
+                  return DMTX_FAILURE;
                   break;
             }
             break;
          case 'f':
-            options->format = *optarg;
-
-            if(options->format != 'n' && options->format != 'p' &&
-                  options->format != 'm') {
-               fprintf(stdout, "Invalid output format \"%c\"\n", options->format);
-               return DMTXUTIL_ERROR;
+            opt->format = *optarg;
+            if(opt->format != 'p' && opt->format != 'm' &&
+                  opt->format != 'a' && opt->format != 'c') {
+               fprintf(stdout, "Invalid output format \"%c\"\n", opt->format);
+               return DMTX_FAILURE;
             }
-
             break;
          case 'o':
-            options->outputPath = optarg;
-            break;
-         case 'p':
-            options->preview = *optarg;
-
-            if(options->preview != 'n' && options->preview != 'a' &&
-                  options->preview != 'c') {
-               fprintf(stdout, "Invalid preview format \"%c\"\n", options->preview);
-               return DMTXUTIL_ERROR;
-            }
-
+            opt->outputPath = optarg;
             break;
          case 'r':
-            err = StringToInt(&(options->rotate), optarg, &ptr);
-            if(err != DMTXUTIL_SUCCESS || *ptr != '\0')
+            err = StringToInt(&(opt->rotate), optarg, &ptr);
+            if(err != DMTX_SUCCESS || *ptr != '\0')
                FatalError(1, _("Invalid rotation angle specified \"%s\""), optarg);
             break;
          case 's':
             /* Determine correct barcode size and/or shape */
             if(*optarg == 's') {
-               options->sizeIdx = DMTX_SYMBOL_SQUARE_AUTO;
+               opt->sizeIdx = DmtxSymbolSquareAuto;
             }
             else if(*optarg == 'r') {
-               options->sizeIdx = DMTX_SYMBOL_RECT_AUTO;
+               opt->sizeIdx = DmtxSymbolRectAuto;
             }
             else {
                for(i = 0; i < DMTX_SYMBOL_SQUARE_COUNT + DMTX_SYMBOL_RECT_COUNT; i++) {
                   if(strncmp(optarg, symbolSizes[i], 8) == 0) {
-                     options->sizeIdx = i;
+                     opt->sizeIdx = i;
                      break;
                   }
                }
                if(i == DMTX_SYMBOL_SQUARE_COUNT + DMTX_SYMBOL_RECT_COUNT)
-                  return DMTXUTIL_ERROR;
+                  return DMTX_FAILURE;
             }
-
             break;
          case 'v':
-            options->verbose = 1;
+            opt->verbose = 1;
             break;
          case 'M':
-            options->mosaic = 1;
+            opt->mosaic = 1;
             break;
          case 'R':
-            err = StringToInt(&(options->dpi), optarg, &ptr);
-            if(err != DMTXUTIL_SUCCESS || options->dpi <= 0 || *ptr != '\0')
+            err = StringToInt(&(opt->dpi), optarg, &ptr);
+            if(err != DMTX_SUCCESS || opt->dpi <= 0 || *ptr != '\0')
                FatalError(1, _("Invalid dpi specified \"%s\""), optarg);
             break;
          case 'V':
@@ -318,37 +286,32 @@ HandleArgs(UserOptions *options, int *argcp, char **argvp[], DmtxEncode *encode)
             exit(0);
             break;
          default:
-            return DMTXUTIL_ERROR;
+            return DMTX_FAILURE;
             break;
       }
    }
 
-   options->inputPath = (*argvp)[optind];
+   opt->inputPath = (*argvp)[optind];
 
    /* XXX here test for incompatibility between options. For example you
       cannot specify dpi if PNM output is requested */
 
-   return DMTXUTIL_SUCCESS;
+   return DMTX_SUCCESS;
 }
 
 /**
+ *
  *
  */
 static void
-ReadData(UserOptions *options, int *codeBufferSize, unsigned char *codeBuffer)
+ReadData(UserOptions *opt, int *codeBufferSize, unsigned char *codeBuffer)
 {
    int fd;
 
    /* Open file or stdin for reading */
-   if(options->inputPath == NULL) {
-      fd = 0;
-   }
-   else {
-      fd = open(options->inputPath, O_RDONLY);
-      if(fd == -1) {
-         FatalError(1, _("Error while opening file \"%s\""), options->inputPath);
-      }
-   }
+   fd = (opt->inputPath == NULL) ? 0 : open(opt->inputPath, O_RDONLY);
+   if(fd == -1)
+      FatalError(1, _("Error while opening file \"%s\""), opt->inputPath);
 
    /* Read input contents into buffer */
    *codeBufferSize = read(fd, codeBuffer, DMTXWRITE_BUFFER_SIZE);
@@ -361,10 +324,9 @@ ReadData(UserOptions *options, int *codeBufferSize, unsigned char *codeBuffer)
 }
 
 /**
- * Display program usage and exit with received status.
- *
- * @param status error code returned to OS
- * @return       void
+ * @brief  Display program usage and exit with received status.
+ * @param  status error code returned to OS
+ * @return void
  */
 static void
 ShowUsage(int status)
@@ -383,31 +345,31 @@ Example: echo -n 123456 | %s -o message.png\n\
 \n\
 OPTIONS:\n"), programName, programName);
       fprintf(stdout, _("\
-  -c, --color=COLOR          barcode color (not implemented)\n\
-  -b, --bg-color=COLOR       background color (not implemented)\n\
-  -d, --module=NUM           module size (in pixels)\n\
-  -m, --margin=NUM           margin size (in pixels)\n\
-  -e, --encoding=[bfactxe8]  encodation scheme\n\
-        b = Best optimized   best possible optimization (beta)\n\
-        f = Fast optimized   basic optimization (not implemented)\n\
-        a = ASCII  [default] ASCII standard & extended\n\
-        c = C40              digits and uppercase\n\
-        t = Text             digits and lowercase\n\
-        x = X12              ANSI X12 EDI\n\
-        e = EDIFACT          ASCII values 32-94\n\
-        8 = Base 256         all byte values 0-255\n\
-  -f, --format=[npm]         image output format\n\
-        n = None             do not create barcode image\n\
-        p = PNG    [default] PNG image\n\
-        m = PNM              PNM image\n\
-  -o, --output=FILE          path of file containing barcode image output\n\
-  -p, --preview=[ac]         print preview of barcode data to STDOUT\n\
-        a = ASCII            ASCII-art representation\n\
-        c = Codewords        list data and error codewords\n\
-  -r, --rotate=DEGREES       rotation angle (degrees)\n\
-  -s, --symbol-size=SIZE     symbol size in Rows x Cols\n\
+  -c, --color=COLOR           barcode color (not implemented)\n\
+  -b, --bg-color=COLOR        background color (not implemented)\n\
+  -d, --module=NUM            module size (in pixels)\n\
+  -m, --margin=NUM            margin size (in pixels)\n\
+  -e, --encoding=[bfactxe8]   encodation scheme; optimize for:\n\
+        b = Best optimized    best possible optimization (beta)\n\
+        f = Fast optimized    basic optimization (not implemented)\n\
+        a = ASCII  [default]  ASCII standard & extended\n\
+        c = C40               digits and uppercase\n\
+        t = Text              digits and lowercase\n\
+        x = X12               ANSI X12 EDI\n\
+        e = EDIFACT           ASCII values 32-94\n\
+        8 = Base 256          all byte values 0-255\n"));
+      fprintf(stdout, _("\
+  -f, --format=[pmac]         image output format\n\
+        p = PNG    [default]  PNG image\n\
+        m = PNM               PNM image\n\
+        a = ASCII             ASCII art barcode (preview only)\n\
+        c = Codewords         Codeword listing  (preview only)\n\
+  -o, --output=FILE           output filename (default standard output)\n\
+  -r, --rotate=DEGREES        rotation angle (degrees)\n"));
+      fprintf(stdout, _("\
+  -s, --symbol-size=SIZE      symbol size in Rows x Cols\n\
         Automatic SIZE options:\n\
-            s = Auto square  [default]\n\
+            s = Auto square   [default]\n\
             r = Auto rectangle\n\
         Manually specified SIZE options for square symbols:\n\
             10x10,   12x12,   14x14,   16x16,   18x18,   20x20,\n\
@@ -415,12 +377,13 @@ OPTIONS:\n"), programName, programName);
             44x44,   48x48,   52x52,   64x64,   72x72,   80x80,\n\
             88x88,   96x96, 104x104, 120x120, 132x132, 144x144\n\
         Manually specified SIZE options for rectangular symbols:\n\
-             8x18,    8x32,   12x26,   12x36,   16x36,   16x48\n\
-  -M, --mosaic               create non-standard Data Mosaic barcode\n\
-  -R, --resolution=NUM       set image print resolution (dpi)\n\
-  -v, --verbose              use verbose messages\n\
-  -V, --version              print version information\n\
-      --help                 display this help and exit\n"));
+             8x18,    8x32,   12x26,   12x36,   16x36,   16x48\n"));
+      fprintf(stdout, _("\
+  -M, --mosaic                create non-standard Data Mosaic barcode\n\
+  -R, --resolution=NUM        set image print resolution (dpi)\n\
+  -v, --verbose               use verbose messages\n\
+  -V, --version               print version information\n\
+      --help                  display this help and exit\n"));
       fprintf(stdout, _("\nReport bugs to <mike@dragonflylogic.com>.\n"));
    }
 
@@ -428,19 +391,22 @@ OPTIONS:\n"), programName, programName);
 }
 
 /**
+ *
  *
  */
 static void
-WriteImagePng(UserOptions *options, DmtxEncode *encode)
+WriteImagePng(UserOptions *opt, DmtxEncode *enc)
 {
    FILE *fp;
    int row;
+   int width, height;
    png_structp pngPtr;
    png_infop infoPtr;
    png_bytepp rowPointers;
    png_uint_32 pixelsPerMeter;
 
-   fp = fopen(options->outputPath, "wb");
+   /* Open file or stdin for writing */
+   fp = (opt->outputPath == NULL) ? stdout : fopen(opt->outputPath, "wb");
    if(fp == NULL) {
       perror(programName);
       exit(3);
@@ -449,53 +415,57 @@ WriteImagePng(UserOptions *options, DmtxEncode *encode)
    /* Create and initialize the png_struct with the desired error handler functions */
    pngPtr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
    if(pngPtr == NULL) {
-      fclose(fp);
+      if(fp != stdout)
+         fclose(fp);
       perror(programName);
    }
 
    /* Create and initialize image information struct */
    infoPtr = png_create_info_struct(pngPtr);
    if(infoPtr == NULL) {
-      fclose(fp);
+      if(fp != stdout)
+         fclose(fp);
       png_destroy_write_struct(&pngPtr,  png_infopp_NULL);
       perror(programName);
    }
 
    /* Set error handling */
    if(setjmp(png_jmpbuf(pngPtr))) {
-      fclose(fp);
+      if(fp != stdout)
+         fclose(fp);
       png_destroy_write_struct(&pngPtr, &infoPtr);
       perror(programName);
    }
 
+   width = dmtxImageGetProp(enc->image, DmtxPropWidth);
+   height = dmtxImageGetProp(enc->image, DmtxPropHeight);
+
    /* Set up output control using standard streams */
    png_init_io(pngPtr, fp);
 
-   png_set_IHDR(pngPtr, infoPtr, encode->image->width, encode->image->height,
+   png_set_IHDR(pngPtr, infoPtr, width, height,
          8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
          PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
 
-   if(options->dpi > 0) {
-      pixelsPerMeter = (png_uint_32)(39.3700787 * options->dpi + 0.5);
+   if(opt->dpi > 0) {
+      pixelsPerMeter = (png_uint_32)(39.3700787 * opt->dpi + 0.5);
       png_set_pHYs(pngPtr, infoPtr, pixelsPerMeter, pixelsPerMeter, PNG_RESOLUTION_METER);
    }
 
-   rowPointers = (png_bytepp)png_malloc(pngPtr, sizeof(png_bytep) * encode->image->height);
+   rowPointers = (png_bytepp)png_malloc(pngPtr, sizeof(png_bytep) * height);
    if(rowPointers == NULL) {
       perror(programName);
    }
 
    /* This copy reverses row order top-to-bottom so image coordinate system
       corresponds with normal "right-handed" 2D space */
-   for(row = 0; row < encode->image->height; row++) {
+   for(row = 0; row < height; row++) {
       rowPointers[row] = (png_bytep)png_malloc(pngPtr, png_get_rowbytes(pngPtr, infoPtr));
 
-      assert(png_get_rowbytes(pngPtr, infoPtr) == encode->image->width * sizeof(DmtxRgb));
+      assert(png_get_rowbytes(pngPtr, infoPtr) == width * sizeof(DmtxRgb));
 
       /* Flip rows top-to-bottom to account for PNM "top-left" origin */
-      memcpy(rowPointers[row],
-            encode->image->pxl + ((encode->image->height - row - 1) * encode->image->width),
-            encode->image->width * sizeof(DmtxRgb));
+      memcpy(rowPointers[row], enc->image->pxl + (row * width), width * sizeof(DmtxRgb));
    }
 
    png_set_rows(pngPtr, infoPtr, rowPointers);
@@ -505,47 +475,57 @@ WriteImagePng(UserOptions *options, DmtxEncode *encode)
    /* Clean up after the write, and free any memory allocated */
    png_destroy_write_struct(&pngPtr, &infoPtr);
 
-   for(row = 0; row < encode->image->height; row++) {
+   for(row = 0; row < height; row++) {
       png_free(pngPtr, rowPointers[row]);
    }
    png_free(pngPtr, rowPointers);
    rowPointers = NULL;
 
-   fclose(fp);
+   if(fp != stdout)
+      fclose(fp);
 }
 
 /**
+ *
  *
  */
 static void
-WriteImagePnm(UserOptions *options, DmtxEncode *encode)
+WriteImagePnm(UserOptions *opt, DmtxEncode *enc)
 {
    int row, col;
+   int width, height;
    FILE *fp;
+   DmtxRgb rgb;
 
-   /* Flip rows top-to-bottom to account for PNM "top-left" origin */
-   fp = fopen(options->outputPath, "wb");
+   fp = (opt->outputPath == NULL) ? stdout : fopen(opt->outputPath, "wb");
    if(fp == NULL) {
       perror(programName);
       exit(3);
    }
 
-   fprintf(fp, "P6 %d %d 255 ", encode->image->width, encode->image->height);
-   for(row = 0; row < encode->image->height; row++) {
-      for(col = 0; col < encode->image->width; col++) {
-         fputc(encode->image->pxl[(encode->image->height - row - 1) * encode->image->width + col][0], fp);
-         fputc(encode->image->pxl[(encode->image->height - row - 1) * encode->image->width + col][1], fp);
-         fputc(encode->image->pxl[(encode->image->height - row - 1) * encode->image->width + col][2], fp);
+   width = dmtxImageGetProp(enc->image, DmtxPropWidth);
+   height = dmtxImageGetProp(enc->image, DmtxPropHeight);
+
+   /* Flip rows top-to-bottom to account for PNM "top-left" origin */
+   fprintf(fp, "P6 %d %d 255 ", width, height);
+   for(row = height - 1; row >= 0; row--) {
+      for(col = 0; col < width; col++) {
+         dmtxImageGetRgb(enc->image, col, row, rgb);
+         fwrite(rgb, sizeof(char), 3, fp);
       }
    }
-   fclose(fp);
+
+   if(fp != stdout) {
+      fclose(fp);
+   }
 }
 
 /**
+ *
  *
  */
 static void
-WriteAsciiBarcode(DmtxEncode *encode)
+WriteAsciiBarcode(DmtxEncode *enc)
 {
    int symbolRow, symbolCol;
    int moduleOnAll;
@@ -555,11 +535,11 @@ WriteAsciiBarcode(DmtxEncode *encode)
    fputc('\n', stdout);
 
    /* ASCII prints from top to bottom */
-   for(symbolRow = encode->region.symbolRows - 1; symbolRow >= 0; symbolRow--) {
+   for(symbolRow = enc->region.symbolRows - 1; symbolRow >= 0; symbolRow--) {
 
       fputs("    ", stdout);
-      for(symbolCol = 0; symbolCol < encode->region.symbolCols; symbolCol++) {
-         fputs((dmtxSymbolModuleStatus(encode->message, encode->region.sizeIdx, symbolRow, symbolCol) &
+      for(symbolCol = 0; symbolCol < enc->region.symbolCols; symbolCol++) {
+         fputs((dmtxSymbolModuleStatus(enc->message, enc->region.sizeIdx, symbolRow, symbolCol) &
                moduleOnAll) ? "XX" : "  ", stdout);
       }
       fputs("\n", stdout);
@@ -569,17 +549,18 @@ WriteAsciiBarcode(DmtxEncode *encode)
 }
 
 /**
+ *
  *
  */
 static void
-WriteCodewords(DmtxEncode *encode)
+WriteCodewords(DmtxEncode *enc)
 {
    int i, dataWordLength;
 
-   dataWordLength = dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, encode->region.sizeIdx);
+   dataWordLength = dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, enc->region.sizeIdx);
 
-   for(i = 0; i < encode->message->codeSize; i++) {
+   for(i = 0; i < enc->message->codeSize; i++) {
       fprintf(stdout, "%c:%03d\n", (i < dataWordLength) ?
-            'd' : 'e', encode->message->code[i]);
+            'd' : 'e', enc->message->code[i]);
    }
 }
